"""DSM Security data."""

from __future__ import annotations

from typing import TypedDict

from synology_dsm.api import SynoBaseApi

SecurityCategory = TypedDict(
    "SecurityCategory",
    {
        "category": str,
        "fail": "dict[str, int]",
        "failSeverity": str,
        "progress": int,
        "runningItem": str,
        "total": int,
        "waitNum": int,
    },
)


class SecurityDataType(TypedDict):
    """Data type."""

    items: dict[str, SecurityCategory]
    lastScanTime: str  # noqa: N815
    startTime: str  # noqa: N815
    success: bool
    sysProgress: int  # noqa: N815
    sysStatus: str  # noqa: N815


class SynoCoreSecurity(SynoBaseApi[SecurityDataType]):
    """Class containing Security data."""

    API_KEY = "SYNO.Core.SecurityScan.Status"

    async def update(self) -> None:
        """Updates security data."""
        raw_data = await self._dsm.get(self.API_KEY, "system_get")
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            self._data = data

    @property
    def checks(self) -> dict[str, SecurityCategory]:
        """Gets the checklist by check category."""
        return self._data["items"]

    @property
    def last_scan_time(self) -> str:
        """Gets the last scan time."""
        return self._data["lastScanTime"]

    @property
    def start_time(self) -> str:
        """Gets the start time (if in progress)."""
        return self._data["startTime"]

    @property
    def success(self) -> bool:
        """Gets the last scan success."""
        return self._data["success"]

    @property
    def progress(self) -> int:
        """Gets the scan progress.

        Returns: 100 if finished
        """
        return self._data["sysProgress"]

    @property
    def status(self) -> str:
        """Gets the last scan status.

        Possible values: safe, danger, info, outOfDate, risk, warning.
        """
        return self._data["sysStatus"]

    @property
    def status_by_check(self) -> dict[str, str]:
        """Gets the last scan status per check."""
        status = {}
        for category in self.checks:
            status[category] = self.checks[category]["failSeverity"]
        return status



"""Data models for Synology Photos Module."""

from __future__ import annotations

from dataclasses import dataclass


@dataclass
class SynoPhotosAlbum:
    """Representation of an Synology Photos Album."""

    album_id: int
    name: str
    item_count: int
    passphrase: str


@dataclass
class SynoPhotosItem:
    """Representation of an Synology Photos Item."""

    item_id: int
    item_type: str
    file_name: str
    file_size: str
    thumbnail_cache_key: str
    thumbnail_size: str
    is_shared: bool
    passphrase: str



"""DSM 5 SYNO.DSM.Info data."""

DSM_5_DSM_INFORMATION_DS410J = {
    "data": {
        "codepage": "enu",
        "model": "DS410j",
        "ram": 128,
        "serial": "A3G7N00628",
        "temperature": 52,
        "temperature_warn": False,
        "time": "Mon Apr 13 18:26:27 2020",
        "uptime": 7077254,
        "version": "5967",
        "version_string": "DSM 5.2-5967 Update 9",
    },
    "success": True,
}

DSM_5_DSM_INFORMATION_DS3615XS = {
    "data": {
        "codepage": "rus",
        "model": "DS3615xs",
        "ram": 6144,
        "serial": "B3J4N01003",
        "temperature": 40,
        "time": "Sat Apr 25 20:21:57 2020",
        "uptime": 3897,
        "version": "5967",
        "version_string": "DSM 5.2-5967 Update 9",
    },
    "success": True,
}

DSM_5_DSM_INFORMATION = DSM_5_DSM_INFORMATION_DS3615XS



"""DSM 6 SYNO.DownloadStation.Task data."""

DSM_6_DOWNLOAD_STATION_TASK_LIST = {
    "data": {
        "offset": 0,
        "tasks": [
            {
                "additional": {
                    "detail": {
                        "completed_time": 0,
                        "connected_leechers": 0,
                        "connected_peers": 0,
                        "connected_seeders": 0,
                        "create_time": 1550089068,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 1592549339,
                        "total_peers": 0,
                        "total_pieces": 1239,
                        "unzip_password": "",
                        "uri": "magnet:?xt=urn:btih:1234ABCD1234ABCD1234ABCD1234ABCD1234ABCD&dn=1234",
                        "waiting_seconds": 0,
                    },
                    "file": [
                        {
                            "filename": "INFO.nfo",
                            "index": 0,
                            "priority": "low",
                            "size": 1335,
                            "size_downloaded": 0,
                            "wanted": True,
                        },
                        {
                            "filename": "My super movie 2 2022.mkv",
                            "index": 1,
                            "priority": "normal",
                            "size": 1591087515,
                            "size_downloaded": 0,
                            "wanted": True,
                        },
                        {
                            "filename": "My super movie 2 sample.mkv",
                            "index": 2,
                            "priority": "normal",
                            "size": 2754524,
                            "size_downloaded": 0,
                            "wanted": False,
                        },
                        {
                            "filename": "My super movie 2021.mkv",
                            "index": 3,
                            "priority": "normal",
                            "size": 1155085341,
                            "size_downloaded": 0,
                            "wanted": True,
                        },
                        {
                            "filename": "My super movie 2021 sample.mkv",
                            "index": 4,
                            "priority": "normal",
                            "size": 4359701,
                            "size_downloaded": 0,
                            "wanted": False,
                        },
                        {
                            "filename": "My super movie 3 2023.mkv",
                            "index": 5,
                            "priority": "normal",
                            "size": 1288819263,
                            "size_downloaded": 0,
                            "wanted": True,
                        },
                        {
                            "filename": "My super movie 3 sample.mkv",
                            "index": 6,
                            "priority": "normal",
                            "size": 3077684,
                            "size_downloaded": 0,
                            "wanted": False,
                        },
                        {
                            "filename": "My super movie 4 2031.mkv",
                            "index": 7,
                            "priority": "normal",
                            "size": 1149397942,
                            "size_downloaded": 0,
                            "wanted": True,
                        },
                        {
                            "filename": "My super movie 4 sample.mkv",
                            "index": 8,
                            "priority": "normal",
                            "size": 2023179,
                            "size_downloaded": 0,
                            "wanted": False,
                        },
                    ],
                },
                "id": "dbid_86",
                "size": 5196586484,
                "status": "downloading",
                "title": "My super movie Complete 2021-2031",
                "type": "bt",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 0,
                        "connected_leechers": 0,
                        "connected_peers": 0,
                        "connected_seeders": 0,
                        "create_time": 1551214114,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 1592549348,
                        "total_peers": 0,
                        "total_pieces": 948,
                        "unzip_password": "",
                        "uri": "magnet:?xt=urn:btih:1234ABCD1234ABCD1234ABCD1234ABCD1234ABCD&dn=1234",
                        "waiting_seconds": 0,
                    },
                    "file": [
                        {
                            "filename": "Blade Swipper 1984.mkv",
                            "index": 0,
                            "priority": "normal",
                            "size": 1986298376,
                            "size_downloaded": 1602519560,
                            "wanted": True,
                        }
                    ],
                },
                "id": "dbid_164",
                "size": 1986298376,
                "status": "downloading",
                "title": "Blade Swipper 1984.mkv",
                "type": "bt",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 0,
                        "connected_leechers": 0,
                        "connected_peers": 50,
                        "connected_seeders": 50,
                        "create_time": 1585435581,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 1592549349,
                        "total_peers": 0,
                        "total_pieces": 0,
                        "unzip_password": "",
                        "uri": "magnet:?xt=urn:btih:1234ABCD1234ABCD1234ABCD1234ABCD1234ABCD&dn=1234",
                        "waiting_seconds": 0,
                    },
                    "file": [],
                },
                "id": "dbid_486",
                "size": 0,
                "status": "downloading",
                "title": "The falling State",
                "type": "bt",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 0,
                        "connected_leechers": 0,
                        "connected_peers": 1,
                        "connected_seeders": 1,
                        "create_time": 1591562665,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 1591563597,
                        "total_peers": 0,
                        "total_pieces": 1494,
                        "unzip_password": "",
                        "uri": "magnet:?xt=urn:btih:1234ABCD1234ABCD1234ABCD1234ABCD1234ABCD&dn=1234",
                        "waiting_seconds": 0,
                    }
                },
                "id": "dbid_518",
                "size": 391580448,
                "status": "paused",
                "title": "Welcome to the North.mkv",
                "type": "bt",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 1591565351,
                        "connected_leechers": 0,
                        "connected_peers": 0,
                        "connected_seeders": 0,
                        "create_time": 1591563606,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 172800,
                        "started_time": 1592601577,
                        "total_peers": 0,
                        "total_pieces": 5466,
                        "unzip_password": "",
                        "uri": "magnet:?xt=urn:btih:1234ABCD1234ABCD1234ABCD1234ABCD1234ABCD&dn=1234",
                        "waiting_seconds": 0,
                    }
                },
                "id": "dbid_522",
                "size": 5731285821,
                "status": "finished",
                "title": "Birds of Pokémon.mkv",
                "type": "bt",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 1591566799,
                        "connected_leechers": 0,
                        "connected_peers": 0,
                        "connected_seeders": 0,
                        "create_time": 1591566523,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 1591566696,
                        "total_peers": 0,
                        "total_pieces": 0,
                        "unzip_password": "",
                        "uri": "https://1fichier.com/?1234ABCD1234ABCD1234&af=22123",
                        "waiting_seconds": 0,
                    }
                },
                "id": "dbid_531",
                "size": 2811892495,
                "status": "finished",
                "title": "1234ABCD1234ABCD1234",
                "type": "https",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 0,
                        "connected_leechers": 0,
                        "connected_peers": 0,
                        "connected_seeders": 0,
                        "create_time": 1591566903,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 0,
                        "total_peers": 0,
                        "total_pieces": 0,
                        "unzip_password": "",
                        "uri": "https://1fichier.com/?123ABC123ABC123ABC12",
                        "waiting_seconds": 0,
                    }
                },
                "id": "dbid_533",
                "size": 0,
                "status": "error",
                "status_extra": {"error_detail": "unknown"},
                "title": "?123ABC123ABC123ABC12",
                "type": "https",
                "username": "Test_useR",
            },
            {
                "additional": {
                    "detail": {
                        "completed_time": 0,
                        "connected_leechers": 0,
                        "connected_peers": 0,
                        "connected_seeders": 0,
                        "create_time": 1592605687,
                        "destination": "Folder/containing/downloads",
                        "seedelapsed": 0,
                        "started_time": 1592605731,
                        "total_peers": 0,
                        "total_pieces": 0,
                        "unzip_password": "",
                        "uri": "https://1fichier.com/?123ABC123ABC123ABC12",
                        "waiting_seconds": 0,
                    }
                },
                "id": "dbid_549",
                "size": 0,
                "status": "error",
                "status_extra": {"error_detail": "broken_link"},
                "title": "123ABC123ABC123ABC12",
                "type": "https",
                "username": "Test_useR",
            },
        ],
        "total": 8,
    },
    "success": True,
}



"""DSM 6 SYNO.Core.Upgrade data."""

DSM_6_CORE_UPGRADE_FALSE = {"data": {"update": {"available": False}}, "success": True}
DSM_6_CORE_UPGRADE_TRUE = {
    "data": {
        "update": {
            "available": True,
            "reboot": "now",
            "restart": "some",
            "type": "nano",
            "version": "DSM 6.2.3-25426 Update 2",
            "version_details": {
                "buildnumber": 25426,
                "major": 6,
                "micro": 3,
                "minor": 2,
                "nano": 2,
                "os_name": "DSM",
            },
        }
    },
    "success": True,
}



"""DSM 7 SYNO.FileStation.List data."""

DSM_7_FILE_STATION_FOLDERS = {
    "data": {
        "offset": 0,
        "shares": [
            {
                "additional": {
                    "mount_point_type": "",
                    "owner": {"gid": 0, "group": "root", "uid": 0, "user": "root"},
                    "perm": {
                        "acl": {
                            "append": True,
                            "del": True,
                            "exec": True,
                            "read": True,
                            "write": True,
                        },
                        "acl_enable": True,
                        "adv_right": {
                            "disable_download": False,
                            "disable_list": False,
                            "disable_modify": False,
                        },
                        "is_acl_mode": True,
                        "is_share_readonly": False,
                        "posix": 777,
                        "share_right": "RW",
                    },
                    "real_path": "/volume1/backup",
                    "sync_share": False,
                    "time": {
                        "atime": 1736098543,
                        "crtime": 1669466095,
                        "ctime": 1736083030,
                        "mtime": 1733848814,
                    },
                    "volume_status": {
                        "freespace": 1553335107584,
                        "readonly": False,
                        "totalspace": 3821146505216,
                    },
                },
                "isdir": True,
                "name": "backup",
                "path": "/backup",
            },
            {
                "additional": {
                    "mount_point_type": "",
                    "owner": {
                        "gid": 100,
                        "group": "users",
                        "uid": 1046,
                        "user": "hass",
                    },
                    "perm": {
                        "acl": {
                            "append": True,
                            "del": True,
                            "exec": True,
                            "read": True,
                            "write": True,
                        },
                        "acl_enable": True,
                        "adv_right": {
                            "disable_download": False,
                            "disable_list": False,
                            "disable_modify": False,
                        },
                        "is_acl_mode": True,
                        "is_share_readonly": False,
                        "posix": 777,
                        "share_right": "RW",
                    },
                    "real_path": "/volume1/homes/hass",
                    "sync_share": False,
                    "time": {
                        "atime": 1736109199,
                        "crtime": 1643818781,
                        "ctime": 1736109071,
                        "mtime": 1736109071,
                    },
                    "volume_status": {
                        "freespace": 1553335107584,
                        "readonly": False,
                        "totalspace": 3821146505216,
                    },
                },
                "isdir": True,
                "name": "home",
                "path": "/home",
            },
        ],
        "total": 2,
    },
    "success": True,
}
DSM_7_FILE_STATION_FILES = {
    "data": {
        "files": [
            {
                "additional": {
                    "mount_point_type": "",
                    "owner": {
                        "gid": 105733,
                        "group": "SynologyPhotos",
                        "uid": 1046,
                        "user": "hass",
                    },
                    "perm": {
                        "acl": {
                            "append": True,
                            "del": True,
                            "exec": True,
                            "read": True,
                            "write": True,
                        },
                        "is_acl_mode": True,
                        "posix": 711,
                    },
                    "real_path": "/volume1/homes/hass/Photos",
                    "size": 50,
                    "time": {
                        "atime": 1735700476,
                        "crtime": 1723653032,
                        "ctime": 1723653464,
                        "mtime": 1723653464,
                    },
                    "type": "",
                },
                "isdir": True,
                "name": "Photos",
                "path": "/home/Photos",
            },
            {
                "additional": {
                    "mount_point_type": "",
                    "owner": {
                        "gid": 100,
                        "group": "users",
                        "uid": 1046,
                        "user": "hass",
                    },
                    "perm": {
                        "acl": {
                            "append": True,
                            "del": True,
                            "exec": True,
                            "read": True,
                            "write": True,
                        },
                        "is_acl_mode": True,
                        "posix": 711,
                    },
                    "real_path": "/volume1/homes/hass/3e57d06c.tar",
                    "size": 1660753920,
                    "time": {
                        "atime": 1736105132,
                        "crtime": 1736105128,
                        "ctime": 1736105132,
                        "mtime": 1736105132,
                    },
                    "type": "TAR",
                },
                "isdir": False,
                "name": "3e57d06c.tar",
                "path": "/home/3e57d06c.tar",
            },
        ],
        "offset": 0,
        "total": 2,
    },
    "success": True,
}



"""Data models for Synology FileStation Module."""

from __future__ import annotations

from dataclasses import dataclass

# -------------------------------------
# generic additional data
# -------------------------------------


@dataclass
class SynoFileAdditionalOwner:
    """Representation of an Synology FileStation additionl owner data."""

    gid: int
    group: str
    uid: int
    user: str


# -------------------------------------
# shared folder
# -------------------------------------


@dataclass
class SynoFileSharedFolderAdditionalPermission:
    """Representation of an Synology FileStation additionl permission data."""

    acl: dict
    acl_enable: bool
    adv_right: dict
    is_acl_mode: bool
    is_share_readonly: bool
    posix: int
    share_right: str


@dataclass
class SynoFileSharedFolderAdditionalVolumeStatus:
    """Representation of an Synology FileStation additionl permission data."""

    freespace: int
    totalspace: int
    readonly: bool


@dataclass
class SynoFileSharedFolderAdditional:
    """Representation of an Synology FileStation Shared Folder additionl data."""

    mount_point_type: str
    owner: SynoFileAdditionalOwner
    perm: SynoFileSharedFolderAdditionalPermission
    volume_status: SynoFileSharedFolderAdditionalVolumeStatus


@dataclass
class SynoFileSharedFolder:
    """Representation of an Synology FileStation Shared Folder."""

    additional: SynoFileSharedFolderAdditional | None
    is_dir: bool
    name: str
    path: str


# -------------------------------------
# file
# -------------------------------------


@dataclass
class SynoFileFileAdditionalPermission:
    """Representation of an Synology FileStation additionl permission data."""

    acl: dict
    is_acl_mode: bool
    posix: int


@dataclass
class SynoFileFileAdditionalTime:
    """Representation of an Synology FileStation additionl permission data."""

    atime: int
    ctime: int
    crtime: int
    mtime: int


@dataclass
class SynoFileFileAdditional:
    """Representation of an Synology FileStation File additionl data."""

    mount_point_type: str
    owner: SynoFileAdditionalOwner
    perm: SynoFileFileAdditionalPermission
    real_path: str
    size: int
    time: SynoFileFileAdditionalTime
    type: str


@dataclass
class SynoFileFile:
    """Representation of an Synology FileStation File."""

    additional: SynoFileFileAdditional | None
    is_dir: bool
    name: str
    path: str



"""DSM 6 SYNO.DownloadStation.Info data."""

DSM_6_DOWNLOAD_STATION_INFO_INFO = {
    "data": {"is_manager": True, "version": 3543, "version_string": "3.8-3543"},
    "success": True,
}

DSM_6_DOWNLOAD_STATION_INFO_CONFIG = {
    "data": {
        "bt_max_download": 0,
        "bt_max_upload": 800,
        "default_destination": "downloads",
        "emule_default_destination": None,
        "emule_enabled": False,
        "emule_max_download": 0,
        "emule_max_upload": 20,
        "ftp_max_download": 0,
        "http_max_download": 0,
        "nzb_max_download": 0,
        "unzip_service_enabled": False,
    },
    "success": True,
}



"""DSM 6 SYNO.Storage.CGI.Storage data."""

from tests.const import UNIQUE_KEY

DSM_6_STORAGE_STORAGE_DS1819_PLUS_SHR2_8DISKS_1VOL = {
    "data": {
        "disks": [
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sda",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 0,
                "exceed_bad_sector_thr": False,
                "firm": "81.00A81",
                "has_system": True,
                "id": "sda",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 1",
                "model": "WD80EFAX-68KNBN0        ",
                "name": "Drive 1",
                "num_id": 1,
                "order": 1,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "VAHG7V1L",
                "size_total": "8001563222016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 39,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdb",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 0,
                "exceed_bad_sector_thr": False,
                "firm": "81.00A81",
                "has_system": True,
                "id": "sdb",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 2",
                "model": "WD80EFAX-68KNBN0        ",
                "name": "Drive 2",
                "num_id": 2,
                "order": 2,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "VAHGLLPL",
                "size_total": "8001563222016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 40,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdc",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 0,
                "exceed_bad_sector_thr": False,
                "firm": "81.00A81",
                "has_system": True,
                "id": "sdc",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 3",
                "model": "WD80EFAX-68KNBN0        ",
                "name": "Drive 3",
                "num_id": 3,
                "order": 3,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "VAHH38AL",
                "size_total": "8001563222016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 40,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdd",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 0,
                "exceed_bad_sector_thr": False,
                "firm": "81.00A81",
                "has_system": True,
                "id": "sdd",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 4",
                "model": "WD80EFAX-68KNBN0        ",
                "name": "Drive 4",
                "num_id": 4,
                "order": 4,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "VAHGW7VL",
                "size_total": "8001563222016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 41,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sde",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 0,
                "exceed_bad_sector_thr": False,
                "firm": "GX2M",
                "has_system": True,
                "id": "sde",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 5",
                "model": "HDWN180                 ",
                "name": "Drive 5",
                "num_id": 5,
                "order": 5,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "399CK0FUFAVG",
                "size_total": "8001563222016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 36,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "TOSHIBA ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdf",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 0,
                "exceed_bad_sector_thr": False,
                "firm": "GX2M",
                "has_system": True,
                "id": "sdf",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 6",
                "model": "HDWN180                 ",
                "name": "Drive 6",
                "num_id": 6,
                "order": 6,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "399CK0G0FAVG",
                "size_total": "8001563222016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 36,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "TOSHIBA ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdg",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 510,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sdg",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 7",
                "model": "WD40EFRX-68WT0N0        ",
                "name": "Drive 7",
                "num_id": 7,
                "order": 7,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC4E7PD98ZY",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 33,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1819+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdh",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 488,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sdh",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 8",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 8",
                "num_id": 8,
                "order": 8,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K3FP148E",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 31,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
        ],
        "env": {
            "batchtask": {"max_task": 64, "remain_task": 64},
            "bay_number": "8",
            "data_scrubbing": {"sche_enabled": "0", "sche_status": "disabled"},
            "ebox": [],
            "fs_acting": False,
            "isSyncSysPartition": False,
            "is_space_actioning": False,
            "isns": {"address": "", "enabled": False},
            "isns_server": "",
            "max_fs_bytes": "118747255799808",
            "max_fs_bytes_high_end": "219902325555200",
            "model_name": "DS1819+",
            "ram_enough_for_fs_high_end": False,
            "ram_size": 16,
            "ram_size_required": 32,
            "showpooltab": True,
            "status": {"system_crashed": False, "system_need_repair": False},
            "support": {"ebox": True, "raid_cross": True, "sysdef": True},
            "support_fit_fs_limit": True,
            "unique_key": UNIQUE_KEY,
            "volume_full_critical": 0.1,
            "volume_full_warning": 0.2,
        },
        "hotSpareConf": {"cross_repair": True, "disable_repair": []},
        "hotSpares": [],
        "iscsiLuns": [],
        "iscsiTargets": [],
        "ports": [],
        "ssdCaches": [],
        "storagePools": [
            {
                "cacheStatus": "",
                "can_do": {
                    "data_scrubbing": True,
                    "delete": True,
                    "expand_by_disk": 1,
                    "raid_cross": True,
                },
                "container": "internal",
                "desc": "SHR-2",
                "device_type": "shr_with_2_disk_protect",
                "disk_failure_number": 0,
                "disks": ["sda", "sdb", "sdc", "sdd", "sde", "sdf", "sdg", "sdh"],
                "drive_type": 0,
                "id": "reuse_1",
                "is_actioning": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 0,
                "limited_disk_number": 24,
                "maximal_disk_size": "0",
                "minimal_disk_size": "8001457274880",
                "next_schedule_time": 0,
                "num_id": 1,
                "pool_child": [{"id": "volume_1", "size": {"total": "39978088267776"}}],
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "multiple",
                "raids": [
                    {
                        "designedDiskCount": 6,
                        "devices": [
                            {"id": "sdf", "slot": 5, "status": "normal"},
                            {"id": "sde", "slot": 4, "status": "normal"},
                            {"id": "sdd", "slot": 3, "status": "normal"},
                            {"id": "sdc", "slot": 2, "status": "normal"},
                            {"id": "sdb", "slot": 1, "status": "normal"},
                            {"id": "sda", "slot": 0, "status": "normal"},
                        ],
                        "hasParity": True,
                        "minDevSize": "8001457274880",
                        "normalDevCount": 6,
                        "raidPath": "/dev/md3",
                        "raidStatus": 1,
                        "spares": [],
                    },
                    {
                        "designedDiskCount": 8,
                        "devices": [
                            {"id": "sdh", "slot": 7, "status": "normal"},
                            {"id": "sdg", "slot": 6, "status": "normal"},
                            {"id": "sdf", "slot": 5, "status": "normal"},
                            {"id": "sde", "slot": 4, "status": "normal"},
                            {"id": "sdd", "slot": 3, "status": "normal"},
                            {"id": "sdc", "slot": 2, "status": "normal"},
                            {"id": "sdb", "slot": 1, "status": "normal"},
                            {"id": "sda", "slot": 0, "status": "normal"},
                        ],
                        "hasParity": True,
                        "minDevSize": "8001457274880",
                        "normalDevCount": 8,
                        "raidPath": "/dev/md2",
                        "raidStatus": 1,
                        "spares": [],
                    },
                ],
                "scrubbingStatus": "has_not_run_yet",
                "size": {"total": "39978100850688", "used": "39978100850688"},
                "space_path": "/dev/vg1",
                "spares": [],
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": False,
                            "errCode": 768,
                            "stopService": False,
                        },
                        "remove": {
                            "can_do": False,
                            "errCode": 768,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": False,
                            "errCode": 768,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            }
        ],
        "volumes": [
            {
                "atime_checked": False,
                "atime_opt": "relatime",
                "cacheStatus": "",
                "can_do": {"delete": True, "raid_cross": True},
                "container": "internal",
                "desc": "Located on Storage Pool 1, SHR",
                "device_type": "shr_with_2_disk_protect",
                "disk_failure_number": 0,
                "disks": [],
                "drive_type": 0,
                "eppool_used": "0",
                "exist_alive_vdsm": False,
                "fs_type": "btrfs",
                "id": "volume_1",
                "is_acting": False,
                "is_actioning": False,
                "is_inode_full": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 0,
                "limited_disk_number": 24,
                "max_fs_size": "1152921504606846976",
                "next_schedule_time": 0,
                "num_id": 1,
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "multiple",
                "scrubbingStatus": "",
                "size": {
                    "free_inode": "0",
                    "total": "38378964738048",
                    "total_device": "39978088267776",
                    "total_inode": "0",
                    "used": "26724878606336",
                },
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "used_by_gluster": False,
                "vol_path": "/volume1",
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {"can_do": True, "errCode": 0, "stopService": True},
                        "remove": {"can_do": True, "errCode": 0, "stopService": False},
                        "resize": {"can_do": True, "errCode": 0, "stopService": False},
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            }
        ],
    },
    "success": True,
}

DSM_6_STORAGE_STORAGE_DS1515_PLUS_SHR2_10DISKS_1VOL_WITH_EXPANSION = {
    "data": {
        "disks": [
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1515+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sda",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 510,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sda",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 1",
                "model": "WD40EFRX-68WT0N0        ",
                "name": "Drive 1",
                "num_id": 1,
                "order": 1,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC4E2JL3SNU",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 33,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1515+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdb",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 490,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sdb",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 2",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 2",
                "num_id": 2,
                "order": 2,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K0NAV92K",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 30,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1515+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdc",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 510,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sdc",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 3",
                "model": "WD40EFRX-68WT0N0        ",
                "name": "Drive 3",
                "num_id": 3,
                "order": 3,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC4E4NTR3PK",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 31,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1515+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdd",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 510,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sdd",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 4",
                "model": "WD40EFRX-68WT0N0        ",
                "name": "Drive 4",
                "num_id": 4,
                "order": 4,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC4E2CDEDH1",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 32,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS1515+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sde",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 510,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": True,
                "id": "sde",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 5",
                "model": "WD40EFRX-68WT0N0        ",
                "name": "Drive 5",
                "num_id": 5,
                "order": 5,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC4E2HAC6NK",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 30,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 1,
                    "str": "DX513-1",
                    "supportPwrBtnDisable": True,
                    "type": "ebox",
                },
                "device": "/dev/sdga",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 492,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": False,
                "id": "sdga",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 1 (DX513-1)",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 1 (DX513-1)",
                "num_id": 1,
                "order": 1,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K1UZSF9D",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 33,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 1,
                    "str": "DX513-1",
                    "supportPwrBtnDisable": True,
                    "type": "ebox",
                },
                "device": "/dev/sdgb",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 498,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": False,
                "id": "sdgb",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 2 (DX513-1)",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 2 (DX513-1)",
                "num_id": 2,
                "order": 2,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K6ZKJE8Y",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 34,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 1,
                    "str": "DX513-1",
                    "supportPwrBtnDisable": True,
                    "type": "ebox",
                },
                "device": "/dev/sdgc",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 494,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": False,
                "id": "sdgc",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 3 (DX513-1)",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 3 (DX513-1)",
                "num_id": 3,
                "order": 3,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K6AKHYDP",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 35,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 1,
                    "str": "DX513-1",
                    "supportPwrBtnDisable": True,
                    "type": "ebox",
                },
                "device": "/dev/sdgd",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 502,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": False,
                "id": "sdgd",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 4 (DX513-1)",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 4 (DX513-1)",
                "num_id": 4,
                "order": 4,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K7JSL21J",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 35,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 1,
                    "str": "DX513-1",
                    "supportPwrBtnDisable": True,
                    "type": "ebox",
                },
                "device": "/dev/sdge",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 508,
                "exceed_bad_sector_thr": False,
                "firm": "82.00A82",
                "has_system": False,
                "id": "sdge",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 5 (DX513-1)",
                "model": "WD40EFRX-68N32N0        ",
                "name": "Drive 5 (DX513-1)",
                "num_id": 5,
                "order": 5,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-WCC7K6ZKJJ7S",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 34,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
        ],
        "env": {
            "batchtask": {"max_task": 64, "remain_task": 64},
            "bay_number": "5",
            "data_scrubbing": {"sche_enabled": "1", "sche_status": "done"},
            "ebox": [{"id": "sdg", "str": "DX513-1"}],
            "fs_acting": False,
            "isSyncSysPartition": False,
            "is_space_actioning": False,
            "isns": {"address": "", "enabled": False},
            "isns_server": "",
            "max_fs_bytes": "118747255799808",
            "max_fs_bytes_high_end": "219902325555200",
            "model_name": "DS1515+",
            "ram_enough_for_fs_high_end": False,
            "ram_size": 2,
            "ram_size_required": 32,
            "showpooltab": False,
            "status": {"system_crashed": False, "system_need_repair": False},
            "support": {"ebox": True, "raid_cross": True, "sysdef": True},
            "support_fit_fs_limit": True,
            "unique_key": UNIQUE_KEY,
            "volume_full_critical": 0.1,
            "volume_full_warning": 0.2,
        },
        "hotSpareConf": {"cross_repair": True, "disable_repair": []},
        "hotSpares": [],
        "iscsiLuns": [],
        "iscsiTargets": [],
        "ports": [],
        "ssdCaches": [],
        "storagePools": [
            {
                "cacheStatus": "",
                "can_do": {
                    "convert_shr_to_pool": 1,
                    "data_scrubbing": True,
                    "delete": True,
                    "expand_by_disk": 1,
                    "raid_cross": True,
                },
                "container": "cross",
                "deploy_path": "volume_1",
                "desc": "",
                "device_type": "shr_with_2_disk_protect",
                "disk_failure_number": 0,
                "disks": [
                    "sda",
                    "sdb",
                    "sdc",
                    "sdd",
                    "sde",
                    "sdga",
                    "sdgb",
                    "sdgc",
                    "sdgd",
                    "sdge",
                ],
                "drive_type": 0,
                "id": "reuse_1",
                "is_actioning": False,
                "is_scheduled": True,
                "is_writable": True,
                "last_done_time": 1585897389,
                "limited_disk_number": 12,
                "maximal_disk_size": "0",
                "minimal_disk_size": "4000681082880",
                "next_schedule_time": 1588320000,
                "num_id": 1,
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "single",
                "raids": [
                    {
                        "designedDiskCount": 10,
                        "devices": [
                            {"id": "sdge", "slot": 6, "status": "normal"},
                            {"id": "sdgd", "slot": 7, "status": "normal"},
                            {"id": "sdgc", "slot": 8, "status": "normal"},
                            {"id": "sdgb", "slot": 9, "status": "normal"},
                            {"id": "sdga", "slot": 5, "status": "normal"},
                            {"id": "sde", "slot": 4, "status": "normal"},
                            {"id": "sdd", "slot": 3, "status": "normal"},
                            {"id": "sdc", "slot": 2, "status": "normal"},
                            {"id": "sdb", "slot": 1, "status": "normal"},
                            {"id": "sda", "slot": 0, "status": "normal"},
                        ],
                        "hasParity": True,
                        "minDevSize": "4000681082880",
                        "normalDevCount": 10,
                        "raidPath": "/dev/md2",
                        "raidStatus": 1,
                        "spares": [],
                    }
                ],
                "scrubbingStatus": "done",
                "size": {"total": "31966715969536", "used": "31966715969536"},
                "space_path": "/dev/vg1000/lv",
                "spares": [],
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {"can_do": True, "errCode": 0, "stopService": True},
                        "remove": {"can_do": True, "errCode": 0, "stopService": False},
                        "resize": {"can_do": True, "errCode": 0, "stopService": False},
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            }
        ],
        "volumes": [
            {
                "atime_checked": False,
                "atime_opt": "relatime",
                "cacheStatus": "",
                "can_do": {
                    "convert_shr_to_pool": 1,
                    "data_scrubbing": True,
                    "delete": True,
                    "expand_by_disk": 1,
                    "raid_cross": True,
                },
                "container": "cross",
                "deploy_path": "volume_1",
                "desc": "",
                "device_type": "shr_with_2_disk_protect",
                "disk_failure_number": 0,
                "disks": [],
                "drive_type": 0,
                "eppool_used": "0",
                "exist_alive_vdsm": False,
                "fs_type": "ext4",
                "id": "volume_1",
                "is_acting": False,
                "is_actioning": False,
                "is_inode_full": False,
                "is_scheduled": True,
                "is_writable": True,
                "last_done_time": 1585897389,
                "limited_disk_number": 12,
                "max_fs_size": "1152921504606846976",
                "next_schedule_time": 1588320000,
                "num_id": 1,
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "single",
                "scrubbingStatus": "done",
                "size": {
                    "free_inode": "974296118",
                    "total": "31714659872768",
                    "total_device": "31966715969536",
                    "total_inode": "975548416",
                    "used": "25419707531264",
                },
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [{"str": "volume_usage_suggestion", "type": "warning"}],
                "timebackup": False,
                "used_by_gluster": False,
                "vol_path": "/volume1",
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {"can_do": True, "errCode": 0, "stopService": True},
                        "remove": {"can_do": True, "errCode": 0, "stopService": False},
                        "resize": {"can_do": True, "errCode": 0, "stopService": False},
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            }
        ],
    },
    "success": True,
}

DSM_6_STORAGE_STORAGE_DS213_PLUS_SHR1_2DISKS_2VOLS = {
    "data": {
        "disks": [
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS213+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sda",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 510,
                "exceed_bad_sector_thr": False,
                "firm": "80.00A80",
                "has_system": True,
                "id": "sda",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 1",
                "model": "WD30E",
                "name": "Drive 1",
                "num_id": 1,
                "order": 1,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "WD-xxxxx",
                "size_total": "3000592982016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 29,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "WDC     ",
            },
            {
                "adv_progress": "",
                "adv_status": "not_support",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS213+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdb",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "",
                "erase_time": 424,
                "exceed_bad_sector_thr": False,
                "firm": "1AN10003",
                "has_system": True,
                "id": "sdb",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 2",
                "model": "HD20xxx",
                "name": "Drive 2",
                "num_id": 2,
                "order": 2,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "S1Uxxxxxxxxxxxx",
                "size_total": "2000398934016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 30,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_2",
                "vendor": "SAMSUNG ",
            },
        ],
        "env": {
            "batchtask": {"max_task": 64, "remain_task": 64},
            "bay_number": "2",
            "data_scrubbing": {"sche_enabled": "0", "sche_status": "disabled"},
            "ebox": [],
            "fs_acting": False,
            "isSyncSysPartition": False,
            "is_space_actioning": False,
            "isns": {"address": "", "enabled": False},
            "isns_server": "",
            "max_fs_bytes": "17592181850112",
            "max_fs_bytes_high_end": "219902325555200",
            "model_name": "DS213+",
            "ram_enough_for_fs_high_end": False,
            "ram_size": 0,
            "ram_size_required": 32,
            "showpooltab": True,
            "status": {"system_crashed": False, "system_need_repair": False},
            "support": {"ebox": True, "raid_cross": False, "sysdef": True},
            "support_fit_fs_limit": False,
            "unique_key": UNIQUE_KEY,
            "volume_full_critical": 0.05,
            "volume_full_warning": 0.1,
        },
        "hotSpareConf": {"cross_repair": True, "disable_repair": []},
        "hotSpares": [],
        "iscsiLuns": [],
        "iscsiTargets": [],
        "ports": [],
        "storagePools": [
            {
                "cacheStatus": "",
                "can_do": {
                    "delete": True,
                    "expand_by_disk": 1,
                    "migrate": {"to_shr2": 3},
                },
                "container": "internal",
                "desc": "Diskgroep 2",
                "device_type": "shr_without_disk_protect",
                "disk_failure_number": 0,
                "disks": ["sdb"],
                "drive_type": 0,
                "id": "reuse_2",
                "is_actioning": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 0,
                "limited_disk_number": 12,
                "maximal_disk_size": "0",
                "minimal_disk_size": "2000292986880",
                "next_schedule_time": 0,
                "num_id": 2,
                "pool_child": [{"id": "volume_2", "size": {"total": "1995435933696"}}],
                "pool_path": "reuse_2",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "multiple",
                "raids": [
                    {
                        "designedDiskCount": 1,
                        "devices": [{"id": "sdb", "slot": 0, "status": "normal"}],
                        "hasParity": False,
                        "minDevSize": "2000292986880",
                        "normalDevCount": 1,
                        "raidPath": "/dev/md3",
                        "raidStatus": 1,
                        "spares": [],
                    }
                ],
                "scrubbingStatus": "has_not_run_yet",
                "size": {"total": "1995448516608", "used": "1995448516608"},
                "space_path": "/dev/vg2",
                "spares": [],
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "remove": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            },
            {
                "cacheStatus": "",
                "can_do": {
                    "delete": True,
                    "expand_by_disk": 1,
                    "migrate": {"to_shr2": 3},
                },
                "container": "internal",
                "desc": "Diskgroep1",
                "device_type": "shr_without_disk_protect",
                "disk_failure_number": 0,
                "disks": ["sda"],
                "drive_type": 0,
                "id": "reuse_1",
                "is_actioning": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 0,
                "limited_disk_number": 12,
                "maximal_disk_size": "0",
                "minimal_disk_size": "3000487034880",
                "next_schedule_time": 0,
                "num_id": 1,
                "pool_child": [{"id": "volume_1", "size": {"total": "2995630637056"}}],
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "multiple",
                "raids": [
                    {
                        "designedDiskCount": 1,
                        "devices": [{"id": "sda", "slot": 0, "status": "normal"}],
                        "hasParity": False,
                        "minDevSize": "3000487034880",
                        "normalDevCount": 1,
                        "raidPath": "/dev/md2",
                        "raidStatus": 1,
                        "spares": [],
                    }
                ],
                "scrubbingStatus": "has_not_run_yet",
                "size": {"total": "2995643219968", "used": "2995643219968"},
                "space_path": "/dev/vg1",
                "spares": [],
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "remove": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            },
        ],
        "volumes": [
            {
                "atime_checked": False,
                "atime_opt": "relatime",
                "cacheStatus": "",
                "can_do": {"delete": True},
                "container": "internal",
                "desc": "Volume_Diskgroep2",
                "device_type": "shr_without_disk_protect",
                "disk_failure_number": 0,
                "disks": [],
                "drive_type": 0,
                "eppool_used": "0",
                "exist_alive_vdsm": False,
                "fs_type": "ext4",
                "id": "volume_2",
                "is_acting": False,
                "is_actioning": False,
                "is_inode_full": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 0,
                "limited_disk_number": 12,
                "max_fs_size": "17577577283584",
                "next_schedule_time": 0,
                "num_id": 2,
                "pool_path": "reuse_2",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "multiple",
                "scrubbingStatus": "",
                "size": {
                    "free_inode": "121689412",
                    "total": "1964124495872",
                    "total_device": "1995435933696",
                    "total_inode": "121798656",
                    "used": "1684179374080",
                },
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "used_by_gluster": False,
                "vol_path": "/volume2",
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "remove": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            },
            {
                "atime_checked": False,
                "atime_opt": "relatime",
                "cacheStatus": "",
                "can_do": {"delete": True},
                "container": "internal",
                "desc": "Volume_Diskgroep1",
                "device_type": "shr_without_disk_protect",
                "disk_failure_number": 0,
                "disks": [],
                "drive_type": 0,
                "eppool_used": "0",
                "exist_alive_vdsm": False,
                "fs_type": "ext4",
                "id": "volume_1",
                "is_acting": False,
                "is_actioning": False,
                "is_inode_full": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 0,
                "limited_disk_number": 12,
                "max_fs_size": "17581339574272",
                "next_schedule_time": 0,
                "num_id": 1,
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "multiple",
                "scrubbingStatus": "",
                "size": {
                    "free_inode": "182794183",
                    "total": "2948623499264",
                    "total_device": "2995630637056",
                    "total_inode": "182845440",
                    "used": "2710796488704",
                },
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [{"str": "volume_usage_suggestion", "type": "warning"}],
                "timebackup": False,
                "used_by_gluster": False,
                "vol_path": "/volume1",
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "remove": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            },
        ],
    },
    "success": True,
}

DSM_6_STORAGE_STORAGE_DS918_PLUS_RAID5_3DISKS_1VOL = {
    "data": {
        "disks": [
            {"id": "test_disk"},
            {
                "adv_progress": "",
                "adv_status": "normal",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS918+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sda",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "ironwolf",
                "erase_time": 448,
                "exceed_bad_sector_thr": False,
                "firm": "SC60",
                "has_system": True,
                "id": "sda",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 1",
                "model": "ST4000VN008-2DR166      ",
                "name": "Drive 1",
                "num_id": 1,
                "order": 1,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "ZDH4LYTS",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 24,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "Seagate",
            },
            {
                "adv_progress": "",
                "adv_status": "normal",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS918+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdb",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "ironwolf",
                "erase_time": 448,
                "exceed_bad_sector_thr": False,
                "firm": "SC60",
                "has_system": True,
                "id": "sdb",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 2",
                "model": "ST4000VN008-2DR166      ",
                "name": "Drive 2",
                "num_id": 2,
                "order": 2,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "ZDH4LS72",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 24,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "Seagate",
            },
            {
                "adv_progress": "",
                "adv_status": "normal",
                "below_remain_life_thr": False,
                "compatibility": "disabled",
                "container": {
                    "order": 0,
                    "str": "DS918+",
                    "supportPwrBtnDisable": False,
                    "type": "internal",
                },
                "device": "/dev/sdc",
                "disable_secera": False,
                "diskType": "SATA",
                "disk_code": "ironwolf",
                "erase_time": 452,
                "exceed_bad_sector_thr": False,
                "firm": "SC60",
                "has_system": True,
                "id": "sdc",
                "ihm_testing": False,
                "is4Kn": False,
                "isSsd": False,
                "isSynoPartition": True,
                "is_erasing": False,
                "longName": "Drive 3",
                "model": "ST4000VN008-2DR166      ",
                "name": "Drive 3",
                "num_id": 3,
                "order": 3,
                "overview_status": "normal",
                "pciSlot": -1,
                "perf_testing": False,
                "portType": "normal",
                "remain_life": -1,
                "remote_info": {"compatibility": "disabled", "unc": 0},
                "serial": "ZDH4LQ1H",
                "size_total": "4000787030016",
                "smart_progress": "",
                "smart_status": "normal",
                "smart_test_limit": 0,
                "smart_testing": False,
                "status": "normal",
                "support": False,
                "temp": 23,
                "testing_progress": "",
                "testing_type": "",
                "tray_status": "join",
                "unc": 0,
                "used_by": "reuse_1",
                "vendor": "Seagate",
            },
        ],
        "env": {
            "batchtask": {"max_task": 64, "remain_task": 64},
            "bay_number": "4",
            "data_scrubbing": {
                "sche_enabled": "0",
                "sche_status": "disabled",
            },
            "ebox": [],
            "fs_acting": False,
            "isSyncSysPartition": False,
            "is_space_actioning": False,
            "isns": {"address": "", "enabled": False},
            "isns_server": "",
            "max_fs_bytes": "118747255799808",
            "max_fs_bytes_high_end": "219902325555200",
            "model_name": "DS918+",
            "ram_enough_for_fs_high_end": False,
            "ram_size": 4,
            "ram_size_required": 32,
            "showpooltab": False,
            "status": {
                "system_crashed": False,
                "system_need_repair": False,
            },
            "support": {
                "ebox": True,
                "raid_cross": True,
                "sysdef": True,
            },
            "support_fit_fs_limit": True,
            "unique_key": UNIQUE_KEY,
            "volume_full_critical": 0.1,
            "volume_full_warning": 0.2,
        },
        "hotSpareConf": {"cross_repair": True, "disable_repair": []},
        "hotSpares": [],
        "iscsiLuns": [],
        "iscsiTargets": [],
        "ports": [],
        "ssdCaches": [],
        "storagePools": [
            {
                "cacheStatus": "",
                "can_do": {
                    "data_scrubbing": True,
                    "delete": True,
                    "expand_by_disk": 1,
                    "migrate": {"to_raid5+spare": "1-1", "to_raid6": 1},
                    "raid_cross": True,
                },
                "container": "internal",
                "deploy_path": "volume_1",
                "desc": "Situé sur Groupe de stockage 1, RAID 5",
                "device_type": "raid_5",
                "disk_failure_number": 0,
                "disks": ["sda", "sdb", "sdc"],
                "drive_type": 0,
                "id": "reuse_1",
                "is_actioning": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 1551201018,
                "limited_disk_number": 24,
                "maximal_disk_size": "0",
                "minimal_disk_size": "4000681164800",
                "next_schedule_time": 0,
                "num_id": 1,
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "single",
                "raids": [
                    {
                        "designedDiskCount": 3,
                        "devices": [
                            {
                                "id": "sdc",
                                "slot": 2,
                                "status": "normal",
                            },
                            {
                                "id": "sdb",
                                "slot": 1,
                                "status": "normal",
                            },
                            {
                                "id": "sda",
                                "slot": 0,
                                "status": "normal",
                            },
                        ],
                        "hasParity": True,
                        "minDevSize": "4000681164800",
                        "normalDevCount": 3,
                        "raidPath": "/dev/md2",
                        "raidStatus": 1,
                        "spares": [],
                    }
                ],
                "scrubbingStatus": "no_action",
                "size": {
                    "total": "7991698522112",
                    "used": "7991698522112",
                },
                "space_path": "/dev/md2",
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": True,
                            "errCode": 0,
                            "stopService": True,
                        },
                        "remove": {
                            "can_do": True,
                            "errCode": 0,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": True,
                            "errCode": 0,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            }
        ],
        "volumes": [
            {"id": "test_volume"},
            {
                "atime_checked": False,
                "atime_opt": "relatime",
                "cacheStatus": "",
                "can_do": {
                    "data_scrubbing": True,
                    "delete": True,
                    "expand_by_disk": 1,
                    "migrate": {"to_raid5+spare": "1-1", "to_raid6": 1},
                    "raid_cross": True,
                },
                "container": "internal",
                "deploy_path": "volume_1",
                "desc": "Situé sur Groupe de stockage 1, RAID 5",
                "device_type": "raid_5",
                "disk_failure_number": 0,
                "disks": [],
                "drive_type": 0,
                "eppool_used": "0",
                "exist_alive_vdsm": False,
                "fs_type": "btrfs",
                "id": "volume_1",
                "is_acting": False,
                "is_actioning": False,
                "is_inode_full": False,
                "is_scheduled": False,
                "is_writable": True,
                "last_done_time": 1551201018,
                "limited_disk_number": 24,
                "max_fs_size": "1152921504606846976",
                "next_schedule_time": 0,
                "num_id": 1,
                "pool_path": "reuse_1",
                "progress": {"percent": "-1", "step": "none"},
                "raidType": "single",
                "scrubbingStatus": "no_action",
                "size": {
                    "free_inode": "0",
                    "total": "7672030584832",
                    "total_device": "7991698522112",
                    "total_inode": "0",
                    "used": "4377452806144",
                },
                "ssd_trim": {"support": "not support"},
                "status": "normal",
                "suggestions": [],
                "timebackup": False,
                "used_by_gluster": False,
                "vol_path": "/volume1",
                "vspace_can_do": {
                    "drbd": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                    "flashcache": {
                        "apply": {
                            "can_do": True,
                            "errCode": 0,
                            "stopService": True,
                        },
                        "remove": {
                            "can_do": True,
                            "errCode": 0,
                            "stopService": False,
                        },
                        "resize": {
                            "can_do": True,
                            "errCode": 0,
                            "stopService": False,
                        },
                    },
                    "snapshot": {
                        "resize": {
                            "can_do": False,
                            "errCode": 53504,
                            "stopService": False,
                        }
                    },
                },
            },
        ],
    },
    "success": True,
}



"""Class to interact with Synology DSM."""

from __future__ import annotations

import asyncio
import logging
import socket
from hashlib import md5
from ipaddress import IPv6Address
from json import JSONDecodeError
from typing import Any, Coroutine, TypedDict
from urllib.parse import quote, urlencode

from aiohttp import (
    ClientError,
    ClientSession,
    ClientTimeout,
    MultipartWriter,
    StreamReader,
    hdrs,
)
from yarl import URL

from .api import SynoBaseApi
from .api.core.external_usb import SynoCoreExternalUSB
from .api.core.security import SynoCoreSecurity
from .api.core.share import SynoCoreShare
from .api.core.system import SynoCoreSystem
from .api.core.upgrade import SynoCoreUpgrade
from .api.core.utilization import SynoCoreUtilization
from .api.download_station import SynoDownloadStation
from .api.dsm.information import SynoDSMInformation
from .api.dsm.network import SynoDSMNetwork
from .api.file_station import SynoFileStation
from .api.photos import SynoPhotos
from .api.storage.storage import SynoStorage
from .api.surveillance_station import SynoSurveillanceStation
from .api.virtual_machine_manager import SynoVirtualMachineManager
from .const import API_AUTH, API_INFO, SENSITIV_PARAMS
from .exceptions import (
    SynologyDSMAPIErrorException,
    SynologyDSMAPINotExistsException,
    SynologyDSMLogin2SAFailedException,
    SynologyDSMLogin2SAForcedException,
    SynologyDSMLogin2SARequiredException,
    SynologyDSMLoginDisabledAccountException,
    SynologyDSMLoginFailedException,
    SynologyDSMLoginInvalidException,
    SynologyDSMLoginPermissionDeniedException,
    SynologyDSMNotLoggedInException,
    SynologyDSMRequestException,
)

_LOGGER = logging.getLogger(__name__)


class ApiType(TypedDict):
    """Synology api info description."""

    maxVersion: int  # noqa: N815
    minVersion: int  # noqa: N815
    path: str


class SynologyDSM:
    """Class containing the main Synology DSM functions."""

    DSM_5_WEIRD_URL_API = [
        SynoStorage.API_KEY,
    ]

    def __init__(
        self,
        session: ClientSession,
        dsm_ip: str,
        dsm_port: int,
        username: str,
        password: str,
        use_https: bool = False,
        timeout: int | ClientTimeout = 10,
        device_token: str | None = None,
        debugmode: bool = False,
    ):
        """Constructor method."""
        self.username = username
        self._password = password
        if isinstance(timeout, ClientTimeout):
            self._aiohttp_timeout = timeout
        else:
            self._aiohttp_timeout = ClientTimeout(total=timeout)
        self._debugmode = debugmode

        # Session
        self._session = session

        # Login
        self._session_id: str | None = None
        self._syno_token: str | None = None
        self._device_token = device_token

        # Services
        self._apis: dict[str, ApiType] = {
            "SYNO.API.Info": {"maxVersion": 1, "minVersion": 1, "path": "query.cgi"}
        }
        self._download: SynoDownloadStation | None = None
        self._external_usb: SynoCoreExternalUSB | None = None
        self._file: SynoFileStation | None = None
        self._information: SynoDSMInformation | None = None
        self._network: SynoDSMNetwork | None = None
        self._photos: SynoPhotos | None = None
        self._security: SynoCoreSecurity | None = None
        self._share: SynoCoreShare | None = None
        self._storage: SynoStorage | None = None
        self._surveillance: SynoSurveillanceStation | None = None
        self._system: SynoCoreSystem | None = None
        self._utilisation: SynoCoreUtilization | None = None
        self._upgrade: SynoCoreUpgrade | None = None
        self._vmm: SynoVirtualMachineManager | None = None

        try:
            IPv6Address(dsm_ip)
            dsm_ip = f"[{dsm_ip}]"
        except ValueError:
            pass

        # Build variables
        if use_https:
            self._base_url = URL(f"https://{dsm_ip}:{dsm_port}")
        else:
            self._base_url = URL(f"http://{dsm_ip}:{dsm_port}")

    def _debuglog(self, message: str) -> None:
        """Outputs message if debug mode is enabled."""
        _LOGGER.debug(message)
        if self._debugmode:
            print("DEBUG: " + message)

    def _is_weird_api_url(self, api: str) -> bool:
        """Returns True if the API URL is not common.

        Common template is nas_base_url/webapi/path?params
        Only handles DSM 5 for now.
        """
        return (
            api in self.DSM_5_WEIRD_URL_API
            and self._information is not None
            and int(self._information.version) < 7321  # < DSM 6
        )

    def _build_url(self, api: str) -> URL:
        if self._is_weird_api_url(api) and api == SynoStorage.API_KEY:
            return self._base_url.join(
                URL("/webman/modules/StorageManager/storagehandler.cgi?")
            )

        return self._base_url.join(URL(f"/webapi/{self.apis[api]['path']}?"))

    async def discover_apis(self) -> None:
        """Retreives available API infos from the NAS."""
        if self._apis.get(API_AUTH):
            return
        data = await self.get(API_INFO, "query")
        if not isinstance(data, dict):
            return

        self._apis = data["data"]

    @property
    def apis(self) -> dict[str, ApiType]:
        """Gets available API infos from the NAS."""
        return self._apis

    async def login(self, otp_code: str | None = None) -> bool:
        """Create a logged session."""
        # First reset the session
        self._debuglog("Creating new session")

        params = {
            "account": self.username,
            "passwd": self._password,
            # "enable_syno_token": "yes",
            "enable_device_token": "yes",  # noqa: S105
            "device_name": socket.gethostname(),
        }

        if otp_code:
            params["otp_code"] = otp_code
        if self._device_token:
            params["device_id"] = self._device_token

        # Request login
        result = await self.get(API_AUTH, "login", params)
        if not isinstance(result, dict):
            return False

        # Handle errors
        if result.get("error"):
            switcher = {
                400: SynologyDSMLoginInvalidException(self.username),
                401: SynologyDSMLoginDisabledAccountException(self.username),
                402: SynologyDSMLoginPermissionDeniedException(self.username),
                403: SynologyDSMLogin2SARequiredException(self.username),
                404: SynologyDSMLogin2SAFailedException(),
                406: SynologyDSMLogin2SAForcedException(self.username),
            }
            raise switcher.get(
                result["error"]["code"],
                SynologyDSMLoginFailedException(result["error"]["code"], self.username),
            )

        # Parse result if valid
        self._session_id = result["data"]["sid"]
        if result["data"].get("synotoken"):
            # Not available on API version < 3
            self._syno_token = result["data"]["synotoken"]
        if result["data"].get("did"):
            # Not available on API version < 6 && device token is given once
            # per device_name
            self._device_token = result["data"]["did"]
        if result["data"].get("device_id"):
            # Not available on API version < 7
            self._device_token = result["data"]["device_id"]
        self._debuglog("Authentication successful, token: " + str(self._session_id))

        if not self._information:
            self._information = SynoDSMInformation(self)
            await self._information.update()

        return bool(result["success"])

    async def logout(self) -> bool:
        """Log out of the session."""
        result = await self.get(API_AUTH, "logout")
        if not isinstance(result, dict):
            return False

        return bool(result["success"])

    @property
    def device_token(self) -> str | None:
        """Gets the device token.

        Used to remember the 2SA access was granted on this device.
        """
        return self._device_token

    async def get(
        self, api: str, method: str, params: dict | None = None, **kwargs: Any
    ) -> bytes | dict | str | StreamReader:
        """Handles API GET request."""
        return await self._request("GET", api, method, params, **kwargs)

    async def post(
        self, api: str, method: str, params: dict | None = None, **kwargs: Any
    ) -> bytes | dict | str | StreamReader:
        """Handles API POST request."""
        return await self._request("POST", api, method, params, **kwargs)

    async def generate_url(
        self,
        api: str,
        method: str,
        params: dict | None = None,
    ) -> str:
        """Generate an url for external usage."""
        url, params, _ = await self._prepare_request(api, method, params)
        return str(URL(url).update_query(params))

    async def _prepare_request(
        self,
        api: str,
        method: str,
        params: dict | None = None,
        **kwargs: Any,
    ) -> tuple[URL, dict, dict]:
        """Prepare the url and parameters for a request."""
        # Discover existing APIs
        if api != API_INFO:
            await self.discover_apis()

        # Check if logged
        if not self._session_id and api not in [API_AUTH, API_INFO]:
            raise SynologyDSMNotLoggedInException

        # Build request params
        if not params:
            params = {}
        params["api"] = api
        params["version"] = 1

        if not self._is_weird_api_url(api):
            # Check if API is available
            if not self.apis.get(api):
                raise SynologyDSMAPINotExistsException(api)
            params["version"] = self.apis[api]["maxVersion"]
            max_version = kwargs.pop("max_version", None)
            if max_version and params["version"] > max_version:
                params["version"] = max_version

        params["method"] = method

        if api == SynoStorage.API_KEY:
            params["action"] = method
        if self._session_id:
            params["_sid"] = self._session_id
        if self._syno_token:
            params["SynoToken"] = self._syno_token

        url = self._build_url(api)

        return (url, params, kwargs)

    async def _request(
        self,
        request_method: str,
        api: str,
        method: str,
        params: dict | None = None,
        retry_once: bool = True,
        raw_response_content: bool = False,
        **kwargs: Any,
    ) -> bytes | dict | str | StreamReader:
        """Handles API request."""
        url, params, kwargs = await self._prepare_request(api, method, params, **kwargs)

        # Request data
        self._debuglog("---------------------------------------------------------")
        self._debuglog("API: " + api)
        self._debuglog("Request Method: " + request_method)
        response = await self._execute_request(
            request_method, url, params, raw_response_content, **kwargs
        )
        self._debuglog("Successful returned data")
        if not raw_response_content:
            self._debuglog("RESPONSE: " + str(response))

        # Handle data errors
        if isinstance(response, dict) and response.get("error") and api != API_AUTH:
            self._debuglog("Session error: " + str(response["error"]["code"]))
            if response["error"]["code"] == 119 and retry_once:
                # Session ID not valid
                # see https://github.com/aerialls/synology-srm/pull/3
                self._session_id = None
                self._syno_token = None
                return await self._request(request_method, api, method, params, False)
            raise SynologyDSMAPIErrorException(
                api, response["error"]["code"], response["error"].get("errors")
            )

        return response

    async def _execute_request(
        self,
        method: str,
        url: URL,
        params: dict,
        raw_response_content: bool = False,
        **kwargs: Any,
    ) -> bytes | dict | str | StreamReader:
        """Function to execute and handle a request."""
        # special handling for spaces in parameters
        # because yarl.URL does encode a space as + instead of %20
        # safe extracted from yarl.URL._QUERY_PART_QUOTER
        query = urlencode(params, safe="?/:@-._~!$'()*,", quote_via=quote)
        url_encoded = url.join(URL(f"?{query}", encoded=True))

        if params.get("api") in [
            SynoFileStation.UPLOAD_API_KEY,
            SynoFileStation.DOWNLOAD_API_KEY,
        ]:
            timeout = ClientTimeout(connect=10.0, total=43200.0)
        else:
            timeout = self._aiohttp_timeout

        try:
            if method == "GET":
                response = await self._session.get(
                    url_encoded, timeout=timeout, **kwargs
                )
            elif (
                method == "POST" and params.get("api") == SynoFileStation.UPLOAD_API_KEY
            ):
                content = kwargs.pop("content")
                path = kwargs.pop("path")
                filename = kwargs.pop("filename")
                create_parents = kwargs.pop("create_parents", None)

                boundary = md5(
                    str(url_encoded).encode("utf-8"), usedforsecurity=False
                ).hexdigest()
                with MultipartWriter("form-data", boundary=boundary) as mp:
                    part = mp.append(path)
                    part.headers.pop(hdrs.CONTENT_TYPE)
                    part.set_content_disposition("form-data", name="path")

                    if create_parents:
                        part = mp.append("true")
                        part.headers.pop(hdrs.CONTENT_TYPE)
                        part.set_content_disposition("form-data", name="create_parents")

                    part = mp.append(content)
                    part.headers.pop(hdrs.CONTENT_TYPE)
                    part.set_content_disposition(
                        "form-data", name="file", filename=filename
                    )
                    part.headers.add(hdrs.CONTENT_TYPE, "application/octet-stream")

                    response = await self._session.post(
                        url_encoded,
                        timeout=timeout,
                        data=mp,
                    )
            elif method == "POST":
                data = {}
                if params is not None:
                    data.update(params)
                data.update(kwargs.pop("data", {}))
                data["mimeType"] = "application/json"
                kwargs["data"] = data
                self._debuglog("POST data: " + str(data))

                response = await self._session.post(
                    url_encoded, timeout=timeout, **kwargs
                )

            # mask sesitive parameters
            if _LOGGER.isEnabledFor(logging.DEBUG) or self._debugmode:
                response_url = response.url  # pylint: disable=E0606
                for param in SENSITIV_PARAMS:
                    if params is not None and params.get(param):
                        response_url = response_url.update_query({param: "*********"})
                self._debuglog("Request url: " + str(response_url))
                self._debuglog(
                    "Request headers: " + str(dict(response.request_info.headers))
                )
                self._debuglog("Response status_code: " + str(response.status))
                self._debuglog("Response headers: " + str(dict(response.headers)))

            if response.status == 200:
                # We got a DSM response
                content_type = response.headers.get("Content-Type", "").split(";")[0]

                if raw_response_content:
                    return response.content

                if content_type in [
                    "application/json",
                    "text/json",
                    "text/plain",  # Can happen with some API
                ]:
                    return dict(await response.json(content_type=content_type))

                if (
                    content_type == "application/octet-stream"
                    or content_type.startswith("image")
                ):
                    return await response.read()

                return await response.text()

            # We got a 400, 401 or 404 ...
            raise ClientError(response)

        except (ClientError, asyncio.TimeoutError, JSONDecodeError) as exp:
            raise SynologyDSMRequestException(exp) from exp

    async def update(
        self, with_information: bool = False, with_network: bool = False
    ) -> None:
        """Updates the various instanced modules."""
        update_methods: list[Coroutine[Any, Any, None]] = []

        if self._download:
            update_methods.append(self._download.update())

        if self._external_usb:
            update_methods.append(self._external_usb.update())

        if self._information and with_information:
            update_methods.append(self._information.update())

        if self._network and with_network:
            update_methods.append(self._network.update())

        if self._security:
            update_methods.append(self._security.update())

        if self._utilisation:
            update_methods.append(self._utilisation.update())

        if self._storage:
            update_methods.append(self._storage.update())

        if self._share:
            update_methods.append(self._share.update())

        if self._surveillance:
            update_methods.append(self._surveillance.update())

        if self._system:
            update_methods.append(self._system.update())

        if self._upgrade:
            update_methods.append(self._upgrade.update())

        if self._vmm:
            update_methods.append(self._vmm.update())

        await asyncio.gather(*update_methods)

    def reset(self, api: SynoBaseApi | str) -> bool:
        """Reset an API to avoid fetching in on update."""
        if isinstance(api, str):
            if api in ("information", SynoDSMInformation.API_KEY):
                return False
            if hasattr(self, "_" + api):
                setattr(self, "_" + api, None)
                return True
            if api == SynoCoreExternalUSB.API_KEY:
                self._external_usb = None
                return True
            if api == SynoFileStation.API_KEY:
                self._file = None
                return True
            if api == SynoCoreSecurity.API_KEY:
                self._security = None
                return True
            if api == SynoCoreShare.API_KEY:
                self._share = None
                return True
            if api == SynoCoreSystem.API_KEY:
                self._system = None
                return True
            if api == SynoCoreUpgrade.API_KEY:
                self._upgrade = None
                return True
            if api == SynoCoreUtilization.API_KEY:
                self._utilisation = None
                return True
            if api == SynoDownloadStation.API_KEY:
                self._download = None
                return True
            if api == SynoPhotos.API_KEY:
                self._photos = None
                return True
            if api == SynoStorage.API_KEY:
                self._storage = None
                return True
            if api == SynoSurveillanceStation.API_KEY:
                self._surveillance = None
                return True
            if api == SynoVirtualMachineManager.API_KEY:
                self._vmm = None
                return True
        if isinstance(api, SynoCoreExternalUSB):
            self._external_usb = None
            return True
        if isinstance(api, SynoFileStation):
            self._file = None
            return True
        if isinstance(api, SynoCoreSecurity):
            self._security = None
            return True
        if isinstance(api, SynoCoreShare):
            self._share = None
            return True
        if isinstance(api, SynoCoreSystem):
            self._system = None
            return True
        if isinstance(api, SynoCoreUpgrade):
            self._upgrade = None
            return True
        if isinstance(api, SynoCoreUtilization):
            self._utilisation = None
            return True
        if isinstance(api, SynoDownloadStation):
            self._download = None
            return True
        if isinstance(api, SynoPhotos):
            self._photos = None
            return True
        if isinstance(api, SynoStorage):
            self._storage = None
            return True
        if isinstance(api, SynoSurveillanceStation):
            self._surveillance = None
            return True
        if isinstance(api, SynoVirtualMachineManager):
            self._vmm = None
            return True
        return False

    @property
    def download_station(self) -> SynoDownloadStation:
        """Gets NAS DownloadStation."""
        if not self._download:
            self._download = SynoDownloadStation(self)
        return self._download

    @property
    def external_usb(self) -> SynoCoreExternalUSB:
        """Gets NAS external USB storage information."""
        if not self._external_usb:
            self._external_usb = SynoCoreExternalUSB(self)
        return self._external_usb

    @property
    def file(self) -> SynoFileStation:
        """Gets NAS FileStation."""
        if not self._file:
            self._file = SynoFileStation(self)
        return self._file

    @property
    def information(self) -> SynoDSMInformation:
        """Gets NAS informations."""
        if not self._information:
            self._information = SynoDSMInformation(self)
        return self._information

    @property
    def network(self) -> SynoDSMNetwork:
        """Gets NAS network informations."""
        if not self._network:
            self._network = SynoDSMNetwork(self)
        return self._network

    @property
    def photos(self) -> SynoPhotos:
        """Gets NAS photos."""
        if not self._photos:
            self._photos = SynoPhotos(self)
        return self._photos

    @property
    def security(self) -> SynoCoreSecurity:
        """Gets NAS security informations."""
        if not self._security:
            self._security = SynoCoreSecurity(self)
        return self._security

    @property
    def share(self) -> SynoCoreShare:
        """Gets NAS shares information."""
        if not self._share:
            self._share = SynoCoreShare(self)
        return self._share

    @property
    def storage(self) -> SynoStorage:
        """Gets NAS storage informations."""
        if not self._storage:
            self._storage = SynoStorage(self)
        return self._storage

    @property
    def surveillance_station(self) -> SynoSurveillanceStation:
        """Gets NAS SurveillanceStation."""
        if not self._surveillance:
            self._surveillance = SynoSurveillanceStation(self)
        return self._surveillance

    @property
    def system(self) -> SynoCoreSystem:
        """Gets NAS system information."""
        if not self._system:
            self._system = SynoCoreSystem(self)
        return self._system

    @property
    def upgrade(self) -> SynoCoreUpgrade:
        """Gets NAS upgrade informations."""
        if not self._upgrade:
            self._upgrade = SynoCoreUpgrade(self)
        return self._upgrade

    @property
    def utilisation(self) -> SynoCoreUtilization:
        """Gets NAS utilisation informations."""
        if not self._utilisation:
            self._utilisation = SynoCoreUtilization(self)
        return self._utilisation

    @property
    def virtual_machine_manager(self) -> SynoVirtualMachineManager:
        """Gets NAS virtual machine manager informations."""
        if not self._vmm:
            self._vmm = SynoVirtualMachineManager(self)
        return self._vmm



"""DSM 6 SYNO.Core.System data."""

DSM_6_CORE_SYSTEM_DS918_PLUS = {
    "data": {
        "cpu_clock_speed": 1500,
        "cpu_cores": "4",
        "cpu_family": "Celeron",
        "cpu_series": "J3455",
        "cpu_vendor": "INTEL",
        "enabled_ntp": True,
        "firmware_date": "2020/07/08",
        "firmware_ver": "DSM 6.2.3-25426 Update 2",
        "model": "DS918+",
        "ntp_server": "time.google.com",
        "ram_size": 4096,
        "sata_dev": [],
        "serial": "1920PDN001501",
        "support_esata": "yes",
        "sys_temp": 40,
        "sys_tempwarn": False,
        "systempwarn": False,
        "temperature_warning": False,
        "time": "2020-10-19 23:33:52",
        "time_zone": "Brussels",
        "time_zone_desc": "(GMT+01:00) Brussels, Copenhagen, Madrid, Paris",
        "up_time": "75:12:9",
        "usb_dev": [
            {
                "cls": "hub",
                "pid": "0612",
                "producer": "Genesys Logic, Inc.",
                "product": "Hub",
                "rev": "92.23",
                "vid": "05e3",
            },
            {
                "cls": "other",
                "pid": "1790",
                "producer": "ASIX Electronics Corp.",
                "product": "AX88179 Gigabit Ethernet",
                "rev": "1.00",
                "vid": "0b95",
            },
            {
                "cls": "hub",
                "pid": "0610",
                "producer": "Genesys Logic, Inc.",
                "product": "4-port hub",
                "rev": "92.23",
                "vid": "05e3",
            },
            {
                "cls": "other",
                "pid": "0200",
                "producer": "Sigma Designs, Inc.",
                "product": "Aeotec Z-Stick Gen5 (ZW090) - UZB",
                "rev": "0.00",
                "vid": "0658",
            },
            {
                "cls": "ups",
                "pid": "0002",
                "producer": "American Power Conversion",
                "product": "Uninterruptible Power Supply",
                "rev": "1.06",
                "vid": "051d",
            },
        ],
    },
    "success": True,
}

DSM_6_CORE_SYSTEM_DS218_PLAY = {
    "data": {
        "cpu_clock_speed": 1400,
        "cpu_cores": "4",
        "cpu_family": "RTD1296",
        "cpu_series": "SoC",
        "cpu_vendor": "Realtek",
        "enabled_ntp": True,
        "firmware_date": "2020/07/14",
        "firmware_ver": "DSM 6.2.3-25426 Update 2",
        "model": "DS218play",
        "ntp_server": "pool.ntp.org",
        "ram_size": 1024,
        "serial": "123456abcdefg",
        "support_esata": "no",
        "sys_temp": 40,
        "sys_tempwarn": False,
        "systempwarn": False,
        "temperature_warning": False,
        "time": "2020-10-16 20:26:58",
        "time_zone": "Amsterdam",
        "time_zone_desc": "(GMT+01:00) Amsterdam, Berlin, Rome, Stockholm, Vienna",
        "up_time": "289:31:54",
        "usb_dev": [
            {
                "cls": "disk",
                "pid": "2621",
                "producer": "Western Digital Technologies, Inc.",
                "product": "Elements 2621",
                "rev": "10.26",
                "vid": "1058",
            }
        ],
    },
    "success": True,
}



"""DSM 6 SYNO.Core.System.Utilization data."""

DSM_6_CORE_UTILIZATION_ERROR_1055 = {
    "error": {
        "code": 1055,
        "errors": {
            "err_key": "",
            "err_line": 883,
            "err_msg": "Transmition failed.",
            "err_session": "",
        },
    },
    "success": False,
}

DSM_6_CORE_UTILIZATION = {
    "data": {
        "cpu": {
            "15min_load": 51,
            "1min_load": 37,
            "5min_load": 33,
            "device": "System",
            "other_load": 3,
            "system_load": 2,
            "user_load": 4,
        },
        "disk": {
            "disk": [
                {
                    "device": "sdc",
                    "display_name": "Drive 3",
                    "read_access": 3,
                    "read_byte": 55261,
                    "type": "internal",
                    "utilization": 12,
                    "write_access": 15,
                    "write_byte": 419425,
                },
                {
                    "device": "sda",
                    "display_name": "Drive 1",
                    "read_access": 3,
                    "read_byte": 63905,
                    "type": "internal",
                    "utilization": 8,
                    "write_access": 14,
                    "write_byte": 414795,
                },
                {
                    "device": "sdb",
                    "display_name": "Drive 2",
                    "read_access": 3,
                    "read_byte": 55891,
                    "type": "internal",
                    "utilization": 10,
                    "write_access": 15,
                    "write_byte": 415658,
                },
            ],
            "total": {
                "device": "total",
                "read_access": 9,
                "read_byte": 175057,
                "utilization": 10,
                "write_access": 44,
                "write_byte": 1249878,
            },
        },
        "lun": [],
        "memory": {
            "avail_real": 156188,
            "avail_swap": 4146316,
            "buffer": 15172,
            "cached": 2764756,
            "device": "Memory",
            "memory_size": 4194304,
            "real_usage": 24,
            "si_disk": 0,
            "so_disk": 0,
            "swap_usage": 6,
            "total_real": 3867268,
            "total_swap": 4415404,
        },
        "network": [
            {"device": "total", "rx": 109549, "tx": 45097},
            {"device": "eth0", "rx": 109549, "tx": 45097},
            {"device": "eth1", "rx": 0, "tx": 0},
        ],
        "space": {
            "total": {
                "device": "total",
                "read_access": 1,
                "read_byte": 27603,
                "utilization": 1,
                "write_access": 23,
                "write_byte": 132496,
            },
            "volume": [
                {
                    "device": "md2",
                    "display_name": "volume1",
                    "read_access": 1,
                    "read_byte": 27603,
                    "utilization": 1,
                    "write_access": 23,
                    "write_byte": 132496,
                }
            ],
        },
        "time": 1585503221,
    },
    "success": True,
}



"""Synology Photos API wrapper."""

from __future__ import annotations

from synology_dsm.api import SynoBaseApi

from .model import SynoPhotosAlbum, SynoPhotosItem


class SynoPhotos(SynoBaseApi):
    """An implementation of a Synology Photos."""

    API_KEY = "SYNO.Foto.*"
    BROWSE_ALBUMS_API_KEY = "SYNO.Foto.Browse.Album"
    BROWSE_ITEM_API_KEY = "SYNO.Foto.Browse.Item"
    DOWNLOAD_API_KEY = "SYNO.Foto.Download"
    DOWNLOAD_FOTOTEAM_API_KEY = "SYNO.FotoTeam.Download"
    SEARCH_API_KEY = "SYNO.Foto.Search.Search"
    THUMBNAIL_API_KEY = "SYNO.Foto.Thumbnail"
    THUMBNAIL_FOTOTEAM_API_KEY = "SYNO.FotoTeam.Thumbnail"
    BROWSE_ITEM_FOTOTEAM_API_KEY = "SYNO.FotoTeam.Browse.Item"

    async def get_albums(
        self, offset: int = 0, limit: int = 100
    ) -> list[SynoPhotosAlbum] | None:
        """Get a list of all albums."""
        albums: list[SynoPhotosAlbum] = []
        raw_data = await self._dsm.get(
            self.BROWSE_ALBUMS_API_KEY,
            "list",
            {"offset": offset, "limit": limit, "category": "normal_share_with_me"},
        )
        if not isinstance(raw_data, dict) or (data := raw_data.get("data")) is None:
            return None

        for album in data["list"]:
            albums.append(
                SynoPhotosAlbum(
                    album["id"],
                    album["name"],
                    album["item_count"],
                    album["passphrase"],
                )
            )
        return albums

    def _raw_data_to_items(  # noqa: S107
        self, raw_data: dict, passphrase: str = ""
    ) -> list[SynoPhotosItem] | None:
        """Parse the raw data response to a list of photo items."""
        items: list[SynoPhotosItem] = []
        if (data := raw_data.get("data")) is None:
            return None

        for item in data["list"]:
            if item["additional"]["thumbnail"]["xl"] == "ready":
                size = "xl"
            elif item["additional"]["thumbnail"]["m"] == "ready":
                size = "m"
            else:
                size = "sm"

            items.append(
                SynoPhotosItem(
                    item["id"],
                    item["type"],
                    item["filename"],
                    item["filesize"],
                    item["additional"]["thumbnail"]["cache_key"],
                    size,
                    item["owner_user_id"] == 0,
                    passphrase,
                )
            )
        return items

    async def get_items_from_album(
        self, album: SynoPhotosAlbum, offset: int = 0, limit: int = 100
    ) -> list[SynoPhotosItem] | None:
        """Get a list of all items from given album."""
        params = {
            "offset": offset,
            "limit": limit,
            "additional": '["thumbnail"]',
        }
        if album.passphrase:
            params["passphrase"] = album.passphrase
        else:
            params["album_id"] = album.album_id

        raw_data = await self._dsm.get(
            self.BROWSE_ITEM_API_KEY,
            "list",
            params,
        )
        if not isinstance(raw_data, dict):
            return None
        return self._raw_data_to_items(raw_data, album.passphrase)

    async def get_items_from_shared_space(
        self, offset: int = 0, limit: int = 100
    ) -> list[SynoPhotosItem] | None:
        """Get a list of all items from the shared space."""
        raw_data = await self._dsm.get(
            self.BROWSE_ITEM_FOTOTEAM_API_KEY,
            "list",
            {
                "offset": offset,
                "limit": limit,
                "additional": '["thumbnail"]',
            },
        )
        if not isinstance(raw_data, dict):
            return None
        return self._raw_data_to_items(raw_data)

    async def get_items_from_search(
        self, search_string: str, offset: int = 0, limit: int = 100
    ) -> list[SynoPhotosItem] | None:
        """Get a list of all items matching the keyword."""
        raw_data = await self._dsm.get(
            self.SEARCH_API_KEY,
            "list_item",
            {
                "keyword": search_string,
                "offset": offset,
                "limit": limit,
                "additional": '["thumbnail"]',
            },
        )
        if not isinstance(raw_data, dict):
            return None
        return self._raw_data_to_items(raw_data)

    async def download_item(self, item: SynoPhotosItem) -> bytes | None:
        """Download the given item."""
        download_api = self.DOWNLOAD_API_KEY
        if item.is_shared:
            download_api = self.DOWNLOAD_FOTOTEAM_API_KEY

        params = {
            "unit_id": f"[{item.item_id}]",
            "cache_key": item.thumbnail_cache_key,
        }

        if item.passphrase:
            params["passphrase"] = item.passphrase

        raw_data = await self._dsm.get(
            download_api,
            "download",
            params,
        )
        if isinstance(raw_data, bytes):
            return raw_data
        return None

    async def download_item_thumbnail(self, item: SynoPhotosItem) -> bytes | None:
        """Download the given items thumbnail."""
        download_api = self.THUMBNAIL_API_KEY
        if item.is_shared:
            download_api = self.THUMBNAIL_FOTOTEAM_API_KEY

        params = {
            "id": item.item_id,
            "cache_key": item.thumbnail_cache_key,
            "size": item.thumbnail_size,
            "type": "unit",
        }

        if item.passphrase:
            params["passphrase"] = item.passphrase

        raw_data = await self._dsm.get(
            download_api,
            "get",
            params,
        )
        if isinstance(raw_data, bytes):
            return raw_data
        return None

    async def get_item_thumbnail_url(self, item: SynoPhotosItem) -> str:
        """Get the url of given items thumbnail."""
        download_api = self.THUMBNAIL_API_KEY
        if item.is_shared:
            download_api = self.THUMBNAIL_FOTOTEAM_API_KEY

        params = {
            "id": item.item_id,
            "cache_key": item.thumbnail_cache_key,
            "size": item.thumbnail_size,
            "type": "unit",
        }

        if item.passphrase:
            params["passphrase"] = item.passphrase

        return await self._dsm.generate_url(
            download_api,
            "get",
            params,
        )



"""Synology DSM tests."""

# pylint: disable=protected-access
import pytest

from synology_dsm.const import API_AUTH
from synology_dsm.exceptions import SynologyDSMLogin2SARequiredException

from . import (
    VALID_HOST,
    VALID_HTTPS,
    VALID_OTP,
    VALID_PASSWORD,
    VALID_PORT,
    VALID_USER_2SA,
    SynologyDSMMock,
)
from .const import DEVICE_TOKEN, SESSION_ID, SYNO_TOKEN


class TestSynologyDSM6:
    """SynologyDSM 6 test cases."""

    @pytest.mark.asyncio
    async def test_login(self, dsm_6):
        """Test login."""
        assert await dsm_6.login()
        assert dsm_6.apis.get(API_AUTH)
        assert dsm_6._session_id == SESSION_ID
        assert dsm_6._syno_token == SYNO_TOKEN

    @pytest.mark.asyncio
    async def test_login_2sa(self):
        """Test login with 2SA."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            VALID_USER_2SA,
            VALID_PASSWORD,
            VALID_HTTPS,
        )

        with pytest.raises(SynologyDSMLogin2SARequiredException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.API.Auth"
        assert error_value["code"] == 403
        assert error_value["reason"] == "One time password not specified"
        assert (
            error_value["details"]
            == "Two-step authentication required for account: valid_user_2sa"
        )

        assert await dsm.login(VALID_OTP)

        assert dsm._session_id == SESSION_ID
        assert dsm._syno_token == SYNO_TOKEN
        assert dsm._device_token == DEVICE_TOKEN
        assert dsm.device_token == DEVICE_TOKEN

    @pytest.mark.asyncio
    async def test_login_2sa_new_session(self):
        """Test login with 2SA and a new session with granted device."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            VALID_USER_2SA,
            VALID_PASSWORD,
            VALID_HTTPS,
            device_token=DEVICE_TOKEN,
        )
        assert await dsm.login()

        assert dsm._session_id == SESSION_ID
        assert dsm._syno_token == SYNO_TOKEN
        assert dsm._device_token == DEVICE_TOKEN
        assert dsm.device_token == DEVICE_TOKEN

    @pytest.mark.asyncio
    async def test_information(self, dsm_6):
        """Test information."""
        assert await dsm_6.login()
        assert dsm_6.information
        await dsm_6.information.update()
        assert dsm_6.information.model == "DS918+"
        assert dsm_6.information.ram == 4096
        assert dsm_6.information.serial == "1920PDN001501"
        assert dsm_6.information.temperature == 40
        assert not dsm_6.information.temperature_warn
        assert dsm_6.information.uptime == 155084
        assert dsm_6.information.version == "24922"
        assert dsm_6.information.version_string == "DSM 6.2.2-24922 Update 4"
        assert dsm_6.information.awesome_version == "6.2.2.4"

    @pytest.mark.asyncio
    async def test_network(self, dsm_6):
        """Test network."""
        assert await dsm_6.login()
        assert dsm_6.network
        await dsm_6.network.update()
        assert dsm_6.network.dns
        assert dsm_6.network.gateway
        assert dsm_6.network.hostname
        assert dsm_6.network.interfaces
        assert dsm_6.network.interface("eth0")
        assert dsm_6.network.interface("eth1")
        assert dsm_6.network.macs
        assert dsm_6.network.workgroup

    @pytest.mark.asyncio
    async def test_security(self, dsm_6):
        """Test security, safe status."""
        assert await dsm_6.login()
        assert dsm_6.security
        await dsm_6.security.update()
        assert dsm_6.security.checks
        assert dsm_6.security.last_scan_time
        assert not dsm_6.security.start_time  # Finished scan
        assert dsm_6.security.success
        assert dsm_6.security.progress
        assert dsm_6.security.status == "safe"
        assert dsm_6.security.status_by_check
        assert dsm_6.security.status_by_check["malware"] == "safe"
        assert dsm_6.security.status_by_check["network"] == "safe"
        assert dsm_6.security.status_by_check["securitySetting"] == "safe"
        assert dsm_6.security.status_by_check["systemCheck"] == "safe"
        assert dsm_6.security.status_by_check["update"] == "safe"
        assert dsm_6.security.status_by_check["userInfo"] == "safe"

    @pytest.mark.asyncio
    async def test_security_error(self, dsm_6):
        """Test security, outOfDate status."""
        assert await dsm_6.login()
        dsm_6.error = True
        assert dsm_6.security
        await dsm_6.security.update()
        assert dsm_6.security.checks
        assert dsm_6.security.last_scan_time
        assert not dsm_6.security.start_time  # Finished scan
        assert dsm_6.security.success
        assert dsm_6.security.progress
        assert dsm_6.security.status == "outOfDate"
        assert dsm_6.security.status_by_check
        assert dsm_6.security.status_by_check["malware"] == "safe"
        assert dsm_6.security.status_by_check["network"] == "safe"
        assert dsm_6.security.status_by_check["securitySetting"] == "safe"
        assert dsm_6.security.status_by_check["systemCheck"] == "safe"
        assert dsm_6.security.status_by_check["update"] == "outOfDate"
        assert dsm_6.security.status_by_check["userInfo"] == "safe"

    @pytest.mark.asyncio
    async def test_shares(self, dsm_6):
        """Test shares."""
        assert await dsm_6.login()
        assert dsm_6.share
        await dsm_6.share.update()
        assert dsm_6.share.shares
        for share_uuid in dsm_6.share.shares_uuids:
            assert dsm_6.share.share_name(share_uuid)
            assert dsm_6.share.share_path(share_uuid)
            assert dsm_6.share.share_recycle_bin(share_uuid) is not None
            assert dsm_6.share.share_size(share_uuid) is not None
            assert dsm_6.share.share_size(share_uuid, human_readable=True)

        assert (
            dsm_6.share.share_name("2ee6c06a-8766-48b5-013d-63b18652a393")
            == "test_share"
        )
        assert (
            dsm_6.share.share_path("2ee6c06a-8766-48b5-013d-63b18652a393") == "/volume1"
        )
        assert (
            dsm_6.share.share_recycle_bin("2ee6c06a-8766-48b5-013d-63b18652a393")
            is True
        )
        assert (
            dsm_6.share.share_size("2ee6c06a-8766-48b5-013d-63b18652a393")
            == 3.790251876432216e19
        )
        assert (
            dsm_6.share.share_size("2ee6c06a-8766-48b5-013d-63b18652a393", True)
            == "32.9Eb"
        )

    @pytest.mark.asyncio
    async def test_system(self, dsm_6):
        """Test system."""
        assert await dsm_6.login()
        assert dsm_6.system
        await dsm_6.system.update()
        assert dsm_6.system.cpu_clock_speed
        assert dsm_6.system.cpu_cores
        assert dsm_6.system.cpu_family
        assert dsm_6.system.cpu_series
        assert dsm_6.system.firmware_ver
        assert dsm_6.system.model
        assert dsm_6.system.ram_size
        assert dsm_6.system.serial
        assert dsm_6.system.sys_temp
        assert dsm_6.system.time
        assert dsm_6.system.time_zone
        assert dsm_6.system.time_zone_desc
        assert dsm_6.system.up_time
        for usb_dev in dsm_6.system.usb_dev:
            assert usb_dev.get("cls")
            assert usb_dev.get("pid")
            assert usb_dev.get("producer")
            assert usb_dev.get("product")
            assert usb_dev.get("rev")
            assert usb_dev.get("vid")

    @pytest.mark.asyncio
    async def test_upgrade(self, dsm_6):
        """Test upgrade."""
        assert await dsm_6.login()
        assert dsm_6.upgrade
        await dsm_6.upgrade.update()
        assert dsm_6.upgrade.update_available
        assert dsm_6.upgrade.available_version == "DSM 6.2.3-25426 Update 2"
        assert dsm_6.upgrade.reboot_needed == "now"
        assert dsm_6.upgrade.service_restarts == "some"
        assert dsm_6.upgrade.available_version_details == {
            "buildnumber": 25426,
            "major": 6,
            "micro": 3,
            "minor": 2,
            "nano": 2,
            "os_name": "DSM",
        }

    @pytest.mark.asyncio
    async def test_storage(self, dsm_6):
        """Test storage roots."""
        assert await dsm_6.login()
        assert dsm_6.storage
        await dsm_6.storage.update()
        assert dsm_6.storage.disks
        assert dsm_6.storage.env
        assert dsm_6.storage.storage_pools
        assert dsm_6.storage.volumes

    @pytest.mark.asyncio
    async def test_storage_raid_volumes(self, dsm_6):
        """Test RAID storage volumes."""
        assert await dsm_6.login()
        await dsm_6.storage.update()
        # Basics
        assert dsm_6.storage.volumes_ids
        for volume_id in dsm_6.storage.volumes_ids:
            if volume_id == "test_volume":
                continue
            assert dsm_6.storage.volume_status(volume_id)
            assert dsm_6.storage.volume_device_type(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id, True)
            assert dsm_6.storage.volume_size_used(volume_id)
            assert dsm_6.storage.volume_size_used(volume_id, True)
            assert dsm_6.storage.volume_percentage_used(volume_id)
            assert dsm_6.storage.volume_disk_temp_avg(volume_id)
            assert dsm_6.storage.volume_disk_temp_max(volume_id)

        # Existing volume
        assert dsm_6.storage.volume_status("volume_1") == "normal"
        assert dsm_6.storage.volume_device_type("volume_1") == "raid_5"
        assert dsm_6.storage.volume_size_total("volume_1") == 7672030584832
        assert dsm_6.storage.volume_size_total("volume_1", True) == "7.0Tb"
        assert dsm_6.storage.volume_size_used("volume_1") == 4377452806144
        assert dsm_6.storage.volume_size_used("volume_1", True) == "4.0Tb"
        assert dsm_6.storage.volume_percentage_used("volume_1") == 57.1
        assert dsm_6.storage.volume_disk_temp_avg("volume_1") == 24.0
        assert dsm_6.storage.volume_disk_temp_max("volume_1") == 24

        # Non existing volume
        assert not dsm_6.storage.volume_status("not_a_volume")
        assert not dsm_6.storage.volume_device_type("not_a_volume")
        assert not dsm_6.storage.volume_size_total("not_a_volume")
        assert not dsm_6.storage.volume_size_total("not_a_volume", True)
        assert not dsm_6.storage.volume_size_used("not_a_volume")
        assert not dsm_6.storage.volume_size_used("not_a_volume", True)
        assert not dsm_6.storage.volume_percentage_used("not_a_volume")
        assert not dsm_6.storage.volume_disk_temp_avg("not_a_volume")
        assert not dsm_6.storage.volume_disk_temp_max("not_a_volume")

        # Test volume
        assert dsm_6.storage.volume_status("test_volume") is None
        assert dsm_6.storage.volume_device_type("test_volume") is None
        assert dsm_6.storage.volume_size_total("test_volume") is None
        assert dsm_6.storage.volume_size_total("test_volume", True) is None
        assert dsm_6.storage.volume_size_used("test_volume") is None
        assert dsm_6.storage.volume_size_used("test_volume", True) is None
        assert dsm_6.storage.volume_percentage_used("test_volume") is None
        assert dsm_6.storage.volume_disk_temp_avg("test_volume") is None
        assert dsm_6.storage.volume_disk_temp_max("test_volume") is None

    @pytest.mark.asyncio
    async def test_storage_shr_volumes(self, dsm_6):
        """Test SHR storage volumes."""
        assert await dsm_6.login()
        dsm_6.disks_redundancy = "SHR1"
        await dsm_6.storage.update()

        # Basics
        assert dsm_6.storage.volumes_ids
        for volume_id in dsm_6.storage.volumes_ids:
            if volume_id == "test_volume":
                continue
            assert dsm_6.storage.volume_status(volume_id)
            assert dsm_6.storage.volume_device_type(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id, True)
            assert dsm_6.storage.volume_size_used(volume_id)
            assert dsm_6.storage.volume_size_used(volume_id, True)
            assert dsm_6.storage.volume_percentage_used(volume_id)
            assert dsm_6.storage.volume_disk_temp_avg(volume_id)
            assert dsm_6.storage.volume_disk_temp_max(volume_id)

        # Existing volume
        assert dsm_6.storage.volume_status("volume_1") == "normal"
        assert (
            dsm_6.storage.volume_device_type("volume_1") == "shr_without_disk_protect"
        )
        assert dsm_6.storage.volume_size_total("volume_1") == 2948623499264
        assert dsm_6.storage.volume_size_total("volume_1", True) == "2.7Tb"
        assert dsm_6.storage.volume_size_used("volume_1") == 2710796488704
        assert dsm_6.storage.volume_size_used("volume_1", True) == "2.5Tb"
        assert dsm_6.storage.volume_percentage_used("volume_1") == 91.9
        assert dsm_6.storage.volume_disk_temp_avg("volume_1") == 29.0
        assert dsm_6.storage.volume_disk_temp_max("volume_1") == 29

        assert dsm_6.storage.volume_status("volume_2") == "normal"
        assert (
            dsm_6.storage.volume_device_type("volume_2") == "shr_without_disk_protect"
        )
        assert dsm_6.storage.volume_size_total("volume_2") == 1964124495872
        assert dsm_6.storage.volume_size_total("volume_2", True) == "1.8Tb"
        assert dsm_6.storage.volume_size_used("volume_2") == 1684179374080
        assert dsm_6.storage.volume_size_used("volume_2", True) == "1.5Tb"
        assert dsm_6.storage.volume_percentage_used("volume_2") == 85.7
        assert dsm_6.storage.volume_disk_temp_avg("volume_2") == 30.0
        assert dsm_6.storage.volume_disk_temp_max("volume_2") == 30

        # Non existing volume
        assert not dsm_6.storage.volume_status("not_a_volume")
        assert not dsm_6.storage.volume_device_type("not_a_volume")
        assert not dsm_6.storage.volume_size_total("not_a_volume")
        assert not dsm_6.storage.volume_size_total("not_a_volume", True)
        assert not dsm_6.storage.volume_size_used("not_a_volume")
        assert not dsm_6.storage.volume_size_used("not_a_volume", True)
        assert not dsm_6.storage.volume_percentage_used("not_a_volume")
        assert not dsm_6.storage.volume_disk_temp_avg("not_a_volume")
        assert not dsm_6.storage.volume_disk_temp_max("not_a_volume")

        # Test volume
        assert dsm_6.storage.volume_status("test_volume") is None
        assert dsm_6.storage.volume_device_type("test_volume") is None
        assert dsm_6.storage.volume_size_total("test_volume") is None
        assert dsm_6.storage.volume_size_total("test_volume", True) is None
        assert dsm_6.storage.volume_size_used("test_volume") is None
        assert dsm_6.storage.volume_size_used("test_volume", True) is None
        assert dsm_6.storage.volume_percentage_used("test_volume") is None
        assert dsm_6.storage.volume_disk_temp_avg("test_volume") is None
        assert dsm_6.storage.volume_disk_temp_max("test_volume") is None

    @pytest.mark.asyncio
    async def test_storage_shr2_volumes(self, dsm_6):
        """Test SHR2 storage volumes."""
        assert await dsm_6.login()
        dsm_6.disks_redundancy = "SHR2"
        await dsm_6.storage.update()

        # Basics
        assert dsm_6.storage.volumes_ids
        for volume_id in dsm_6.storage.volumes_ids:
            assert dsm_6.storage.volume_status(volume_id)
            assert dsm_6.storage.volume_device_type(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id, True)
            assert dsm_6.storage.volume_size_used(volume_id)
            assert dsm_6.storage.volume_size_used(volume_id, True)
            assert dsm_6.storage.volume_percentage_used(volume_id)
            assert dsm_6.storage.volume_disk_temp_avg(volume_id)
            assert dsm_6.storage.volume_disk_temp_max(volume_id)

        # Existing volume
        assert dsm_6.storage.volume_status("volume_1") == "normal"
        assert dsm_6.storage.volume_device_type("volume_1") == "shr_with_2_disk_protect"
        assert dsm_6.storage.volume_size_total("volume_1") == 38378964738048
        assert dsm_6.storage.volume_size_total("volume_1", True) == "34.9Tb"
        assert dsm_6.storage.volume_size_used("volume_1") == 26724878606336
        assert dsm_6.storage.volume_size_used("volume_1", True) == "24.3Tb"
        assert dsm_6.storage.volume_percentage_used("volume_1") == 69.6
        assert dsm_6.storage.volume_disk_temp_avg("volume_1") == 37.0
        assert dsm_6.storage.volume_disk_temp_max("volume_1") == 41

    @pytest.mark.asyncio
    async def test_storage_shr2_expansion_volumes(self, dsm_6):
        """Test SHR2 storage with expansion unit volumes."""
        assert await dsm_6.login()
        dsm_6.disks_redundancy = "SHR2_EXPANSION"
        await dsm_6.storage.update()

        # Basics
        assert dsm_6.storage.volumes_ids
        for volume_id in dsm_6.storage.volumes_ids:
            assert dsm_6.storage.volume_status(volume_id)
            assert dsm_6.storage.volume_device_type(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id)
            assert dsm_6.storage.volume_size_total(volume_id, True)
            assert dsm_6.storage.volume_size_used(volume_id)
            assert dsm_6.storage.volume_size_used(volume_id, True)
            assert dsm_6.storage.volume_percentage_used(volume_id)
            assert dsm_6.storage.volume_disk_temp_avg(volume_id)
            assert dsm_6.storage.volume_disk_temp_max(volume_id)

        # Existing volume
        assert dsm_6.storage.volume_status("volume_1") == "normal"
        assert dsm_6.storage.volume_device_type("volume_1") == "shr_with_2_disk_protect"
        assert dsm_6.storage.volume_size_total("volume_1") == 31714659872768
        assert dsm_6.storage.volume_size_total("volume_1", True) == "28.8Tb"
        assert dsm_6.storage.volume_size_used("volume_1") == 25419707531264
        assert dsm_6.storage.volume_size_used("volume_1", True) == "23.1Tb"
        assert dsm_6.storage.volume_percentage_used("volume_1") == 80.2
        assert dsm_6.storage.volume_disk_temp_avg("volume_1") == 33.0
        assert dsm_6.storage.volume_disk_temp_max("volume_1") == 35

    @pytest.mark.asyncio
    async def test_storage_disks(self, dsm_6):
        """Test storage disks."""
        assert await dsm_6.login()
        await dsm_6.storage.update()
        # Basics
        assert dsm_6.storage.disks_ids
        for disk_id in dsm_6.storage.disks_ids:
            if disk_id == "test_disk":
                continue
            assert "Drive" in dsm_6.storage.disk_name(disk_id)
            assert "/dev/" in dsm_6.storage.disk_device(disk_id)
            assert dsm_6.storage.disk_smart_status(disk_id) == "normal"
            assert dsm_6.storage.disk_status(disk_id) == "normal"
            assert not dsm_6.storage.disk_exceed_bad_sector_thr(disk_id)
            assert not dsm_6.storage.disk_below_remain_life_thr(disk_id)
            assert dsm_6.storage.disk_temp(disk_id)

        # Non existing disk
        assert not dsm_6.storage.disk_name("not_a_disk")
        assert not dsm_6.storage.disk_device("not_a_disk")
        assert not dsm_6.storage.disk_smart_status("not_a_disk")
        assert not dsm_6.storage.disk_status("not_a_disk")
        assert not dsm_6.storage.disk_exceed_bad_sector_thr("not_a_disk")
        assert not dsm_6.storage.disk_below_remain_life_thr("not_a_disk")
        assert not dsm_6.storage.disk_temp("not_a_disk")

        # Test disk
        assert dsm_6.storage.disk_name("test_disk") is None
        assert dsm_6.storage.disk_device("test_disk") is None
        assert dsm_6.storage.disk_smart_status("test_disk") is None
        assert dsm_6.storage.disk_status("test_disk") is None
        assert dsm_6.storage.disk_exceed_bad_sector_thr("test_disk") is None
        assert dsm_6.storage.disk_below_remain_life_thr("test_disk") is None
        assert dsm_6.storage.disk_temp("test_disk") is None

    @pytest.mark.asyncio
    async def test_download_station(self, dsm_6):
        """Test DownloadStation."""
        assert await dsm_6.login()
        assert dsm_6.download_station
        assert not dsm_6.download_station.get_all_tasks()

        assert (await dsm_6.download_station.get_info())["data"]["version"]
        assert (await dsm_6.download_station.get_config())["data"][
            "default_destination"
        ]
        assert (await dsm_6.download_station.get_stat())["data"]["speed_download"]
        await dsm_6.download_station.update()
        assert dsm_6.download_station.get_all_tasks()
        assert len(dsm_6.download_station.get_all_tasks()) == 8

        # BT DL
        assert dsm_6.download_station.get_task("dbid_86").status == "downloading"
        assert not dsm_6.download_station.get_task("dbid_86").status_extra
        assert dsm_6.download_station.get_task("dbid_86").type == "bt"
        assert dsm_6.download_station.get_task("dbid_86").additional.get("file")
        assert (
            len(dsm_6.download_station.get_task("dbid_86").additional.get("file")) == 9
        )

        # HTTPS error
        assert dsm_6.download_station.get_task("dbid_549").status == "error"
        assert (
            dsm_6.download_station.get_task("dbid_549").status_extra["error_detail"]
            == "broken_link"
        )
        assert dsm_6.download_station.get_task("dbid_549").type == "https"

    @pytest.mark.asyncio
    async def test_surveillance_station(self, dsm_6):
        """Test SurveillanceStation."""
        dsm_6.with_surveillance = True
        assert await dsm_6.login()
        assert dsm_6.surveillance_station
        assert not dsm_6.surveillance_station.get_all_cameras()

        await dsm_6.surveillance_station.update()
        assert dsm_6.surveillance_station.get_all_cameras()
        assert dsm_6.surveillance_station.get_camera(1)
        assert dsm_6.surveillance_station.get_camera_live_view_path(1)
        assert dsm_6.surveillance_station.get_camera_live_view_path(1, "rtsp")

        # Motion detection
        assert (await dsm_6.surveillance_station.enable_motion_detection(1)).get(
            "success"
        )
        assert (await dsm_6.surveillance_station.disable_motion_detection(1)).get(
            "success"
        )

        # Home mode
        assert await dsm_6.surveillance_station.get_home_mode_status()
        assert await dsm_6.surveillance_station.set_home_mode(False)
        assert await dsm_6.surveillance_station.set_home_mode(True)



"""DSM 5 SYNO.API.Auth data."""

from tests.const import DEVICE_TOKEN, ERROR_AUTH_OTP_NOT_SPECIFIED, SESSION_ID

# No synotoken for an unknown reason
DSM_5_AUTH_LOGIN = {
    "data": {"is_portal_port": False, "sid": SESSION_ID},
    "success": True,
}
DSM_5_AUTH_LOGIN_2SA = ERROR_AUTH_OTP_NOT_SPECIFIED
DSM_5_AUTH_LOGIN_2SA_OTP = {
    "data": {"did": DEVICE_TOKEN, "is_portal_port": False, "sid": SESSION_ID},
    "success": True,
}
